@@ -52,6 +52,13 @@ PG_FUNCTION_INFO_V1(pgrowlocks);
 
 #define NCHARS 32
 
+typedef struct
+{
+	Relation	rel;
+	HeapScanDesc scan;
+	int			ncolumns;
+} MyData;
+
 #define		Atnum_tid		0
 #define		Atnum_xmax		1
 #define		Atnum_ismulti	2
@@ -62,80 +69,77 @@ PG_FUNCTION_INFO_V1(pgrowlocks);
 Datum
 pgrowlocks(PG_FUNCTION_ARGS)
 {
-	text	   *relname = PG_GETARG_TEXT_PP(0);
-	ReturnSetInfo *rsinfo = (ReturnSetInfo *) fcinfo->resultinfo;
-	bool		randomAccess;
-	TupleDesc	tupdesc;
-	Tuplestorestate *tupstore;
-	AttInMetadata *attinmeta;
-	Relation	rel;
-	RangeVar   *relrv;
+	FuncCallContext *funcctx;
 	HeapScanDesc scan;
 	HeapTuple	tuple;
-	MemoryContext oldcontext;
-	AclResult	aclresult;
-	char	  **values;
+	TupleDesc	tupdesc;
+	AttInMetadata *attinmeta;
+	Datum		result;
+	MyData	   *mydata;
+	Relation	rel;
 
-	/* check to see if caller supports us returning a tuplestore */
-	if (rsinfo == NULL || !IsA(rsinfo, ReturnSetInfo))
-		ereport(ERROR,
-				(errcode(ERRCODE_FEATURE_NOT_SUPPORTED),
-				 errmsg("set-valued function called in context that cannot accept a set")));
-	if (!(rsinfo->allowedModes & SFRM_Materialize))
-		ereport(ERROR,
-				(errcode(ERRCODE_SYNTAX_ERROR),
-				 errmsg("materialize mode required, but it is not allowed in this context")));
+	if (SRF_IS_FIRSTCALL())
+	{
+		text	   *relname;
+		RangeVar   *relrv;
+		MemoryContext oldcontext;
+		AclResult	aclresult;
 
-	/* The tupdesc and tuplestore must be created in ecxt_per_query_memory */
-	oldcontext = MemoryContextSwitchTo(rsinfo->econtext->ecxt_per_query_memory);
+		funcctx = SRF_FIRSTCALL_INIT();
+		oldcontext = MemoryContextSwitchTo(funcctx->multi_call_memory_ctx);
 
-	if (get_call_result_type(fcinfo, NULL, &tupdesc) != TYPEFUNC_COMPOSITE)
-		elog(ERROR, "return type must be a row type");
+		/* Build a tuple descriptor for our result type */
+		if (get_call_result_type(fcinfo, NULL, &tupdesc) != TYPEFUNC_COMPOSITE)
+			elog(ERROR, "return type must be a row type");
 
-	randomAccess = (rsinfo->allowedModes & SFRM_Materialize_Random) != 0;
-	tupstore = tuplestore_begin_heap(randomAccess, false, work_mem);
-	rsinfo->returnMode = SFRM_Materialize;
-	rsinfo->setResult = tupstore;
-	rsinfo->setDesc = tupdesc;
+		attinmeta = TupleDescGetAttInMetadata(tupdesc);
+		funcctx->attinmeta = attinmeta;
 
-	MemoryContextSwitchTo(oldcontext);
+		relname = PG_GETARG_TEXT_PP(0);
+		relrv = makeRangeVarFromNameList(textToQualifiedNameList(relname));
+		rel = relation_openrv(relrv, AccessShareLock);
 
-	/* Access the table */
-	relrv = makeRangeVarFromNameList(textToQualifiedNameList(relname));
-	rel = relation_openrv(relrv, AccessShareLock);
+		if (rel->rd_rel->relkind == RELKIND_PARTITIONED_TABLE)
+			ereport(ERROR,
+					(errcode(ERRCODE_WRONG_OBJECT_TYPE),
+					 errmsg("\"%s\" is a partitioned table",
+							RelationGetRelationName(rel)),
+					 errdetail("Partitioned tables do not contain rows.")));
+		else if (rel->rd_rel->relkind != RELKIND_RELATION)
+			ereport(ERROR,
+					(errcode(ERRCODE_WRONG_OBJECT_TYPE),
+					 errmsg("\"%s\" is not a table",
+							RelationGetRelationName(rel))));
 
-	if (rel->rd_rel->relkind == RELKIND_PARTITIONED_TABLE)
-		ereport(ERROR,
-				(errcode(ERRCODE_WRONG_OBJECT_TYPE),
-				 errmsg("\"%s\" is a partitioned table",
-						RelationGetRelationName(rel)),
-				 errdetail("Partitioned tables do not contain rows.")));
-	else if (rel->rd_rel->relkind != RELKIND_RELATION)
-		ereport(ERROR,
-				(errcode(ERRCODE_WRONG_OBJECT_TYPE),
-				 errmsg("\"%s\" is not a table",
-						RelationGetRelationName(rel))));
+		/*
+		 * check permissions: must have SELECT on table or be in
+		 * pg_stat_scan_tables
+		 */
+		aclresult = pg_class_aclcheck(RelationGetRelid(rel), GetUserId(),
+									  ACL_SELECT);
+		if (aclresult != ACLCHECK_OK)
+			aclresult = is_member_of_role(GetUserId(), DEFAULT_ROLE_STAT_SCAN_TABLES) ? ACLCHECK_OK : ACLCHECK_NO_PRIV;
 
-	/*
-	 * check permissions: must have SELECT on table or be in
-	 * pg_stat_scan_tables
-	 */
-	aclresult = pg_class_aclcheck(RelationGetRelid(rel), GetUserId(),
-								  ACL_SELECT);
-	if (aclresult != ACLCHECK_OK)
-		aclresult = is_member_of_role(GetUserId(), DEFAULT_ROLE_STAT_SCAN_TABLES) ? ACLCHECK_OK : ACLCHECK_NO_PRIV;
+		if (aclresult != ACLCHECK_OK)
+			aclcheck_error(aclresult, get_relkind_objtype(rel->rd_rel->relkind),
+						   RelationGetRelationName(rel));
 
-	if (aclresult != ACLCHECK_OK)
-		aclcheck_error(aclresult, ACL_KIND_CLASS,
-					   RelationGetRelationName(rel));
+		scan = heap_beginscan(rel, GetActiveSnapshot(), 0, NULL);
+		mydata = palloc(sizeof(*mydata));
+		mydata->rel = rel;
+		mydata->scan = scan;
+		mydata->ncolumns = tupdesc->natts;
+		funcctx->user_fctx = mydata;
 
-	/* Scan the relation */
-	scan = heap_beginscan(rel, GetActiveSnapshot(), 0, NULL);
+		MemoryContextSwitchTo(oldcontext);
+	}
 
-	attinmeta = TupleDescGetAttInMetadata(tupdesc);
-
-	values = (char **) palloc(tupdesc->natts * sizeof(char *));
+	funcctx = SRF_PERCALL_SETUP();
+	attinmeta = funcctx->attinmeta;
+	mydata = (MyData *) funcctx->user_fctx;
+	scan = mydata->scan;
 
+	/* scan the relation */
 	while ((tuple = heap_getnext(scan, ForwardScanDirection)) != NULL)
 	{
 		HTSU_Result htsu;
@@ -156,6 +160,10 @@ pgrowlocks(PG_FUNCTION_ARGS)
 		 */
 		if (htsu == HeapTupleBeingUpdated)
 		{
+			char	  **values;
+
+			values = (char **) palloc(mydata->ncolumns * sizeof(char *));
+
 			values[Atnum_tid] = (char *) DirectFunctionCall1(tidout,
 															 PointerGetDatum(&tuple->t_self));
 
@@ -280,7 +288,16 @@ pgrowlocks(PG_FUNCTION_ARGS)
 
 			/* build a tuple */
 			tuple = BuildTupleFromCStrings(attinmeta, values);
-			tuplestore_puttuple(tupstore, tuple);
+
+			/* make the tuple into a datum */
+			result = HeapTupleGetDatum(tuple);
+
+			/*
+			 * no need to pfree what we allocated; it's on a short-lived
+			 * memory context anyway
+			 */
+
+			SRF_RETURN_NEXT(funcctx, result);
 		}
 		else
 		{
@@ -289,6 +306,7 @@ pgrowlocks(PG_FUNCTION_ARGS)
 	}
 
 	heap_endscan(scan);
-	heap_close(rel, AccessShareLock);
-	return (Datum) 0;
+	heap_close(mydata->rel, AccessShareLock);
+
+	SRF_RETURN_DONE(funcctx);
 }
