@@ -3,7 +3,7 @@
  * vacuumlo.c
  *	  This removes orphaned large objects from a database.
  *
- * Portions Copyright (c) 1996-2017, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2018, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  *
@@ -21,7 +21,7 @@
 #include <termios.h>
 #endif
 
-#include "catalog/pg_class.h"
+#include "catalog/pg_class_d.h"
 
 #include "fe_utils/connect.h"
 #include "libpq-fe.h"
@@ -245,9 +245,9 @@ vacuumlo(const char *database, const struct _param *param)
 			PQfinish(conn);
 			if (schema != NULL)
 				PQfreemem(schema);
-			if (table != NULL)
+			if (schema != NULL)
 				PQfreemem(table);
-			if (field != NULL)
+			if (schema != NULL)
 				PQfreemem(field);
 			return -1;
 		}
@@ -316,7 +316,7 @@ vacuumlo(const char *database, const struct _param *param)
 
 	deleted = 0;
 
-	do
+	while (1)
 	{
 		res = PQexec(conn, buf);
 		if (PQresultStatus(res) != PGRES_TUPLES_OK)
@@ -354,7 +354,8 @@ vacuumlo(const char *database, const struct _param *param)
 					if (PQtransactionStatus(conn) == PQTRANS_INERROR)
 					{
 						success = false;
-						break;	/* out of inner for-loop */
+						PQclear(res);
+						break;
 					}
 				}
 				else
@@ -392,7 +393,7 @@ vacuumlo(const char *database, const struct _param *param)
 		}
 
 		PQclear(res);
-	} while (success);
+	}
 
 	/*
 	 * That's all folks!
