@@ -3,7 +3,7 @@
  * pg_depend.c
  *	  routines to support manipulation of the pg_depend relation
  *
- * Portions Copyright (c) 1996-2017, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2018, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  *
@@ -125,23 +125,15 @@ recordMultipleDependencies(const ObjectAddress *depender,
 
 /*
  * If we are executing a CREATE EXTENSION operation, mark the given object
- * as being a member of the extension, or check that it already is one.
- * Otherwise, do nothing.
+ * as being a member of the extension.  Otherwise, do nothing.
  *
  * This must be called during creation of any user-definable object type
  * that could be a member of an extension.
  *
- * isReplace must be true if the object already existed, and false if it is
- * newly created.  In the former case we insist that it already be a member
- * of the current extension.  In the latter case we can skip checking whether
- * it is already a member of any extension.
- *
- * Note: isReplace = true is typically used when updating a object in
- * CREATE OR REPLACE and similar commands.  We used to allow the target
- * object to not already be an extension member, instead silently absorbing
- * it into the current extension.  However, this was both error-prone
- * (extensions might accidentally overwrite free-standing objects) and
- * a security hazard (since the object would retain its previous ownership).
+ * If isReplace is true, the object already existed (or might have already
+ * existed), so we must check for a pre-existing extension membership entry.
+ * Passing false is a guarantee that the object is newly created, and so
+ * could not already be a member of any extension.
  */
 void
 recordDependencyOnCurrentExtension(const ObjectAddress *object,
@@ -159,12 +151,6 @@ recordDependencyOnCurrentExtension(const ObjectAddress *object,
 		{
 			Oid			oldext;
 
-			/*
-			 * Side note: these catalog lookups are safe only because the
-			 * object is a pre-existing one.  In the not-isReplace case, the
-			 * caller has most likely not yet done a CommandCounterIncrement
-			 * that would make the new object visible.
-			 */
 			oldext = getExtensionOfObject(object->classId, object->objectId);
 			if (OidIsValid(oldext))
 			{
@@ -178,13 +164,6 @@ recordDependencyOnCurrentExtension(const ObjectAddress *object,
 								getObjectDescription(object),
 								get_extension_name(oldext))));
 			}
-			/* It's a free-standing object, so reject */
-			ereport(ERROR,
-					(errcode(ERRCODE_OBJECT_NOT_IN_PREREQUISITE_STATE),
-					 errmsg("%s is not a member of extension \"%s\"",
-							getObjectDescription(object),
-							get_extension_name(CurrentExtensionObject)),
-					 errdetail("An extension is not allowed to replace an object that it does not own.")));
 		}
 
 		/* OK, record it as a member of CurrentExtensionObject */
@@ -196,49 +175,6 @@ recordDependencyOnCurrentExtension(const ObjectAddress *object,
 	}
 }
 
-/*
- * If we are executing a CREATE EXTENSION operation, check that the given
- * object is a member of the extension, and throw an error if it isn't.
- * Otherwise, do nothing.
- *
- * This must be called whenever a CREATE IF NOT EXISTS operation (for an
- * object type that can be an extension member) has found that an object of
- * the desired name already exists.  It is insecure for an extension to use
- * IF NOT EXISTS except when the conflicting object is already an extension
- * member; otherwise a hostile user could substitute an object with arbitrary
- * properties.
- */
-void
-checkMembershipInCurrentExtension(const ObjectAddress *object)
-{
-	/*
-	 * This is actually the same condition tested in
-	 * recordDependencyOnCurrentExtension; but we want to issue a
-	 * differently-worded error, and anyway it would be pretty confusing to
-	 * call recordDependencyOnCurrentExtension in these circumstances.
-	 */
-
-	/* Only whole objects can be extension members */
-	Assert(object->objectSubId == 0);
-
-	if (creating_extension)
-	{
-		Oid			oldext;
-
-		oldext = getExtensionOfObject(object->classId, object->objectId);
-		/* If already a member of this extension, OK */
-		if (oldext == CurrentExtensionObject)
-			return;
-		/* Else complain */
-		ereport(ERROR,
-				(errcode(ERRCODE_OBJECT_NOT_IN_PREREQUISITE_STATE),
-				 errmsg("%s is not a member of extension \"%s\"",
-						getObjectDescription(object),
-						get_extension_name(CurrentExtensionObject)),
-				 errdetail("An extension may only use CREATE ... IF NOT EXISTS to skip object creation if the conflicting object is one that it already owns.")));
-	}
-}
-
 /*
  * deleteDependencyRecordsFor -- delete all records with given depender
  * classId/objectId.  Returns the number of records deleted.
@@ -549,55 +485,12 @@ getExtensionOfObject(Oid classId, Oid objectId)
 	return result;
 }
 
-/*
- * Return (possibly NIL) list of extensions that the given object depends on
- * in DEPENDENCY_AUTO_EXTENSION mode.
- */
-List *
-getAutoExtensionsOfObject(Oid classId, Oid objectId)
-{
-	List	   *result = NIL;
-	Relation	depRel;
-	ScanKeyData	key[2];
-	SysScanDesc	scan;
-	HeapTuple	tup;
-
-	depRel = heap_open(DependRelationId, AccessShareLock);
-
-	ScanKeyInit(&key[0],
-				Anum_pg_depend_classid,
-				BTEqualStrategyNumber, F_OIDEQ,
-				ObjectIdGetDatum(classId));
-	ScanKeyInit(&key[1],
-				Anum_pg_depend_objid,
-				BTEqualStrategyNumber, F_OIDEQ,
-				ObjectIdGetDatum(objectId));
-
-	scan = systable_beginscan(depRel, DependDependerIndexId, true,
-							  NULL, 2, key);
-
-	while (HeapTupleIsValid((tup = systable_getnext(scan))))
-	{
-		Form_pg_depend depform = (Form_pg_depend) GETSTRUCT(tup);
-
-		if (depform->refclassid == ExtensionRelationId &&
-			depform->deptype == DEPENDENCY_AUTO_EXTENSION)
-			result = lappend_oid(result, depform->refobjid);
-	}
-
-	systable_endscan(scan);
-
-	heap_close(depRel, AccessShareLock);
-
-	return result;
-}
-
 /*
  * Detect whether a sequence is marked as "owned" by a column
  *
  * An ownership marker is an AUTO or INTERNAL dependency from the sequence to the
  * column.  If we find one, store the identity of the owning column
- * into *tableId and *colId and return TRUE; else return FALSE.
+ * into *tableId and *colId and return true; else return false.
  *
  * Note: if there's more than one such pg_depend entry then you get
  * a random one of them returned into the out parameters.  This should
@@ -723,8 +616,8 @@ getOwnedSequence(Oid relid, AttrNumber attnum)
 
 /*
  * get_constraint_index
- *		Given the OID of a unique, primary-key, or exclusion constraint,
- *		return the OID of the underlying index.
+ *		Given the OID of a unique or primary-key constraint, return the
+ *		OID of the underlying unique index.
  *
  * Return InvalidOid if the index couldn't be found; this suggests the
  * given OID is bogus, but we leave it to caller to decide what to do.
@@ -763,14 +656,19 @@ get_constraint_index(Oid constraintId)
 
 		/*
 		 * We assume any internal dependency of an index on the constraint
-		 * must be what we are looking for.  (The relkind test is just
-		 * paranoia; there shouldn't be any such dependencies otherwise.)
+		 * must be what we are looking for.
 		 */
 		if (deprec->classid == RelationRelationId &&
 			deprec->objsubid == 0 &&
-			deprec->deptype == DEPENDENCY_INTERNAL &&
-			get_rel_relkind(deprec->objid) == RELKIND_INDEX)
+			deprec->deptype == DEPENDENCY_INTERNAL)
 		{
+			char		relkind = get_rel_relkind(deprec->objid);
+
+			/* This is pure paranoia; there shouldn't be any such */
+			if (relkind != RELKIND_INDEX &&
+				relkind != RELKIND_PARTITIONED_INDEX)
+				break;
+
 			indexId = deprec->objid;
 			break;
 		}
@@ -784,9 +682,8 @@ get_constraint_index(Oid constraintId)
 
 /*
  * get_index_constraint
- *		Given the OID of an index, return the OID of the owning unique,
- *		primary-key, or exclusion constraint, or InvalidOid if there
- *		is no owning constraint.
+ *		Given the OID of an index, return the OID of the owning unique or
+ *		primary-key constraint, or InvalidOid if no such constraint.
  */
 Oid
 get_index_constraint(Oid indexId)
