@@ -909,7 +909,8 @@ parseqatom(struct vars *v,
 			}
 			/* legal in EREs due to specification botch */
 			NOTE(REG_UPBOTCH);
-			/* fallthrough into case PLAIN */
+			/* fall through into case PLAIN */
+			/* FALLTHROUGH */
 		case PLAIN:
 			onechr(v, v->nextvalue, lp, rp);
 			okcolors(v->nfa, v->cm);
@@ -1154,10 +1155,7 @@ parseqatom(struct vars *v,
 		/* rest of branch can be strung starting from atom->end */
 		s2 = atom->end;
 	}
-	else if (m == 1 && n == 1 &&
-			 (qprefer == 0 ||
-			  (atom->flags & (LONGER | SHORTER | MIXED)) == 0 ||
-			  qprefer == (atom->flags & (LONGER | SHORTER | MIXED))))
+	else if (m == 1 && n == 1)
 	{
 		/* no/vacuous quantifier:  done */
 		EMPTYARC(s, atom->begin);	/* empty prefix */
