@@ -3,7 +3,7 @@
  * ts_locale.c
  *		locale compatibility layer for tsearch
  *
- * Portions Copyright (c) 1996-2017, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2018, PostgreSQL Global Development Group
  *
  *
  * IDENTIFICATION
@@ -21,8 +21,6 @@
 static void tsearch_readline_callback(void *arg);
 
 
-#ifdef USE_WIDE_UPPER_LOWER
-
 /*
  * The reason these functions use a 3-wchar_t output buffer, not 2 as you
  * might expect, is that on Windows "wchar_t" is 16 bits and what we'll be
@@ -97,7 +95,6 @@ t_isprint(const char *ptr)
 
 	return iswprint((wint_t) character[0]);
 }
-#endif							/* USE_WIDE_UPPER_LOWER */
 
 
 /*
@@ -255,17 +252,12 @@ char *
 lowerstr_with_len(const char *str, int len)
 {
 	char	   *out;
-
-#ifdef USE_WIDE_UPPER_LOWER
 	Oid			collation = DEFAULT_COLLATION_OID;	/* TODO */
 	pg_locale_t mylocale = 0;	/* TODO */
-#endif
 
 	if (len == 0)
 		return pstrdup("");
 
-#ifdef USE_WIDE_UPPER_LOWER
-
 	/*
 	 * Use wide char code only when max encoding length > 1 and ctype != C.
 	 * Some operating systems fail with multi-byte encodings and a C locale.
@@ -311,7 +303,6 @@ lowerstr_with_len(const char *str, int len)
 		Assert(wlen < len);
 	}
 	else
-#endif							/* USE_WIDE_UPPER_LOWER */
 	{
 		const char *ptr = str;
 		char	   *outptr;
