@@ -39,7 +39,7 @@
  * empty and be returned to the free page manager, and whole segments can
  * become empty and be returned to the operating system.
  *
- * Portions Copyright (c) 1996-2017, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2018, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  * IDENTIFICATION
@@ -693,16 +693,7 @@ dsa_allocate_extended(dsa_area *area, size_t size, int flags)
 		/* Obtain a span object. */
 		span_pointer = alloc_object(area, DSA_SCLASS_BLOCK_OF_SPANS);
 		if (!DsaPointerIsValid(span_pointer))
-		{
-			/* Raise error unless asked not to. */
-			if ((flags & DSA_ALLOC_NO_OOM) == 0)
-				ereport(ERROR,
-						(errcode(ERRCODE_OUT_OF_MEMORY),
-						 errmsg("out of memory"),
-						 errdetail("Failed on DSA request of size %zu.",
-								   size)));
 			return InvalidDsaPointer;
-		}
 
 		LWLockAcquire(DSA_AREA_LOCK(area), LW_EXCLUSIVE);
 
@@ -799,10 +790,12 @@ dsa_allocate_extended(dsa_area *area, size_t size, int flags)
 	{
 		/* Raise error unless asked not to. */
 		if ((flags & DSA_ALLOC_NO_OOM) == 0)
+		{
 			ereport(ERROR,
 					(errcode(ERRCODE_OUT_OF_MEMORY),
 					 errmsg("out of memory"),
 					 errdetail("Failed on DSA request of size %zu.", size)));
+		}
 		return InvalidDsaPointer;
 	}
 
@@ -1676,14 +1669,13 @@ ensure_active_superblock(dsa_area *area, dsa_area_pool *pool,
 			return false;
 		}
 	}
-	/*
-	 * This shouldn't happen: get_best_segment() or make_new_segment()
-	 * promised that we can successfully allocate npages.
-	 */
 	if (!FreePageManagerGet(segment_map->fpm, npages, &first_page))
-		elog(FATAL,
-			 "dsa_allocate could not find %zu free pages for superblock",
-			 npages);
+	{
+		LWLockRelease(DSA_AREA_LOCK(area));
+		if (size_class != DSA_SCLASS_BLOCK_OF_SPANS)
+			dsa_free(area, span_pointer);
+		return false;
+	}
 	LWLockRelease(DSA_AREA_LOCK(area));
 
 	/* Compute the start of the superblock. */
