@@ -1,7 +1,7 @@
 /*
  * psql - the PostgreSQL interactive terminal
  *
- * Copyright (c) 2000-2017, PostgreSQL Global Development Group
+ * Copyright (c) 2000-2018, PostgreSQL Global Development Group
  *
  * src/bin/psql/help.c
  */
@@ -167,13 +167,14 @@ slashUsage(unsigned short int pager)
 	 * Use "psql --help=commands | wc" to count correctly.  It's okay to count
 	 * the USE_READLINE line even in builds without that.
 	 */
-	output = PageOutput(124, pager ? &(pset.popt.topt) : NULL);
+	output = PageOutput(125, pager ? &(pset.popt.topt) : NULL);
 
 	fprintf(output, _("General\n"));
 	fprintf(output, _("  \\copyright             show PostgreSQL usage and distribution terms\n"));
 	fprintf(output, _("  \\crosstabview [COLUMNS] execute query and display results in crosstab\n"));
 	fprintf(output, _("  \\errverbose            show most recent error message at maximum verbosity\n"));
 	fprintf(output, _("  \\g [FILE] or ;         execute query (and send results to file or |pipe)\n"));
+	fprintf(output, _("  \\gdesc                 describe result of query, without executing it\n"));
 	fprintf(output, _("  \\gexec                 execute query, then execute each value in its result\n"));
 	fprintf(output, _("  \\gset [PREFIX]         execute query and store results in psql variables\n"));
 	fprintf(output, _("  \\gx [FILE]             as \\g, but forces expanded output mode\n"));
@@ -234,7 +235,7 @@ slashUsage(unsigned short int pager)
 	fprintf(output, _("  \\des[+] [PATTERN]      list foreign servers\n"));
 	fprintf(output, _("  \\deu[+] [PATTERN]      list user mappings\n"));
 	fprintf(output, _("  \\dew[+] [PATTERN]      list foreign-data wrappers\n"));
-	fprintf(output, _("  \\df[antw][S+] [PATRN]  list [only agg/normal/trigger/window] functions\n"));
+	fprintf(output, _("  \\df[anptw][S+] [PATRN] list [only agg/normal/procedures/trigger/window] functions\n"));
 	fprintf(output, _("  \\dF[+]  [PATTERN]      list text search configurations\n"));
 	fprintf(output, _("  \\dFd[+] [PATTERN]      list text search dictionaries\n"));
 	fprintf(output, _("  \\dFp[+] [PATTERN]      list text search parsers\n"));
@@ -245,7 +246,7 @@ slashUsage(unsigned short int pager)
 	fprintf(output, _("  \\dL[S+] [PATTERN]      list procedural languages\n"));
 	fprintf(output, _("  \\dm[S+] [PATTERN]      list materialized views\n"));
 	fprintf(output, _("  \\dn[S+] [PATTERN]      list schemas\n"));
-	fprintf(output, _("  \\do[S+] [PATTERN]      list operators\n"));
+	fprintf(output, _("  \\do[S]  [PATTERN]      list operators\n"));
 	fprintf(output, _("  \\dO[S+] [PATTERN]      list collations\n"));
 	fprintf(output, _("  \\dp     [PATTERN]      list table, view, and sequence access privileges\n"));
 	fprintf(output, _("  \\drds [PATRN1 [PATRN2]] list per-database role settings\n"));
@@ -257,7 +258,7 @@ slashUsage(unsigned short int pager)
 	fprintf(output, _("  \\du[S+] [PATTERN]      list roles\n"));
 	fprintf(output, _("  \\dv[S+] [PATTERN]      list views\n"));
 	fprintf(output, _("  \\dx[+]  [PATTERN]      list extensions\n"));
-	fprintf(output, _("  \\dy[+]  [PATTERN]      list event triggers\n"));
+	fprintf(output, _("  \\dy     [PATTERN]      list event triggers\n"));
 	fprintf(output, _("  \\l[+]   [PATTERN]      list databases\n"));
 	fprintf(output, _("  \\sf[+]  FUNCNAME       show a function's definition\n"));
 	fprintf(output, _("  \\sv[+]  VIEWNAME       show a view's definition\n"));
@@ -336,7 +337,7 @@ helpVariables(unsigned short int pager)
 	 * Windows builds currently print one more line than non-Windows builds.
 	 * Using the larger number is fine.
 	 */
-	output = PageOutput(93, pager ? &(pset.popt.topt) : NULL);
+	output = PageOutput(156, pager ? &(pset.popt.topt) : NULL);
 
 	fprintf(output, _("List of specially treated variables\n\n"));
 
@@ -344,67 +345,118 @@ helpVariables(unsigned short int pager)
 	fprintf(output, _("Usage:\n"));
 	fprintf(output, _("  psql --set=NAME=VALUE\n  or \\set NAME VALUE inside psql\n\n"));
 
-	fprintf(output, _("  AUTOCOMMIT         if set, successful SQL commands are automatically committed\n"));
-	fprintf(output, _("  COMP_KEYWORD_CASE  determines the case used to complete SQL key words\n"
-					  "                     [lower, upper, preserve-lower, preserve-upper]\n"));
-	fprintf(output, _("  DBNAME             the currently connected database name\n"));
-	fprintf(output, _("  ECHO               controls what input is written to standard output\n"
-					  "                     [all, errors, none, queries]\n"));
-	fprintf(output, _("  ECHO_HIDDEN        if set, display internal queries executed by backslash commands;\n"
-					  "                     if set to \"noexec\", just show without execution\n"));
-	fprintf(output, _("  ENCODING           current client character set encoding\n"));
-	fprintf(output, _("  FETCH_COUNT        the number of result rows to fetch and display at a time\n"
-					  "                     (default: 0=unlimited)\n"));
-	fprintf(output, _("  HISTCONTROL        controls command history [ignorespace, ignoredups, ignoreboth]\n"));
-	fprintf(output, _("  HISTFILE           file name used to store the command history\n"));
-	fprintf(output, _("  HISTSIZE           max number of commands to store in the command history\n"));
-	fprintf(output, _("  HOST               the currently connected database server host\n"));
-	fprintf(output, _("  IGNOREEOF          number of EOFs needed to terminate an interactive session\n"));
-	fprintf(output, _("  LASTOID            value of the last affected OID\n"));
-	fprintf(output, _("  ON_ERROR_ROLLBACK  if set, an error doesn't stop a transaction (uses implicit savepoints)\n"));
-	fprintf(output, _("  ON_ERROR_STOP      stop batch execution after error\n"));
-	fprintf(output, _("  PORT               server port of the current connection\n"));
-	fprintf(output, _("  PROMPT1            specifies the standard psql prompt\n"));
-	fprintf(output, _("  PROMPT2            specifies the prompt used when a statement continues from a previous line\n"));
-	fprintf(output, _("  PROMPT3            specifies the prompt used during COPY ... FROM STDIN\n"));
-	fprintf(output, _("  QUIET              run quietly (same as -q option)\n"));
-	fprintf(output, _("  SERVER_VERSION_NAME  server's version (short string)\n"));
-	fprintf(output, _("  SERVER_VERSION_NUM   server's version (numeric format)\n"));
-	fprintf(output, _("  SHOW_CONTEXT       controls display of message context fields [never, errors, always]\n"));
-	fprintf(output, _("  SINGLELINE         end of line terminates SQL command mode (same as -S option)\n"));
-	fprintf(output, _("  SINGLESTEP         single-step mode (same as -s option)\n"));
-	fprintf(output, _("  USER               the currently connected database user\n"));
-	fprintf(output, _("  VERBOSITY          controls verbosity of error reports [default, verbose, terse]\n"));
-	fprintf(output, _("  VERSION            psql's version (verbose string)\n"));
-	fprintf(output, _("  VERSION_NAME       psql's version (short string)\n"));
-	fprintf(output, _("  VERSION_NUM        psql's version (numeric format)\n"));
+	fprintf(output, _("  AUTOCOMMIT\n"
+					  "    if set, successful SQL commands are automatically committed\n"));
+	fprintf(output, _("  COMP_KEYWORD_CASE\n"
+					  "    determines the case used to complete SQL key words\n"
+					  "    [lower, upper, preserve-lower, preserve-upper]\n"));
+	fprintf(output, _("  DBNAME\n"
+					  "    the currently connected database name\n"));
+	fprintf(output, _("  ECHO\n"
+					  "    controls what input is written to standard output\n"
+					  "    [all, errors, none, queries]\n"));
+	fprintf(output, _("  ECHO_HIDDEN\n"
+					  "    if set, display internal queries executed by backslash commands;\n"
+					  "    if set to \"noexec\", just show them without execution\n"));
+	fprintf(output, _("  ENCODING\n"
+					  "    current client character set encoding\n"));
+	fprintf(output, _("  ERROR\n"
+					  "    true if last query failed, else false\n"));
+	fprintf(output, _("  FETCH_COUNT\n"
+					  "    the number of result rows to fetch and display at a time (0 = unlimited)\n"));
+	fprintf(output, _("  HISTCONTROL\n"
+					  "    controls command history [ignorespace, ignoredups, ignoreboth]\n"));
+	fprintf(output, _("  HISTFILE\n"
+					  "    file name used to store the command history\n"));
+	fprintf(output, _("  HISTSIZE\n"
+					  "    maximum number of commands to store in the command history\n"));
+	fprintf(output, _("  HOST\n"
+					  "    the currently connected database server host\n"));
+	fprintf(output, _("  IGNOREEOF\n"
+					  "    number of EOFs needed to terminate an interactive session\n"));
+	fprintf(output, _("  LASTOID\n"
+					  "    value of the last affected OID\n"));
+	fprintf(output, _("  LAST_ERROR_MESSAGE\n"
+					  "  LAST_ERROR_SQLSTATE\n"
+					  "    message and SQLSTATE of last error, or empty string and \"00000\" if none\n"));
+	fprintf(output, _("  ON_ERROR_ROLLBACK\n"
+					  "    if set, an error doesn't stop a transaction (uses implicit savepoints)\n"));
+	fprintf(output, _("  ON_ERROR_STOP\n"
+					  "    stop batch execution after error\n"));
+	fprintf(output, _("  PORT\n"
+					  "    server port of the current connection\n"));
+	fprintf(output, _("  PROMPT1\n"
+					  "    specifies the standard psql prompt\n"));
+	fprintf(output, _("  PROMPT2\n"
+					  "    specifies the prompt used when a statement continues from a previous line\n"));
+	fprintf(output, _("  PROMPT3\n"
+					  "    specifies the prompt used during COPY ... FROM STDIN\n"));
+	fprintf(output, _("  QUIET\n"
+					  "    run quietly (same as -q option)\n"));
+	fprintf(output, _("  ROW_COUNT\n"
+					  "    number of rows returned or affected by last query, or 0\n"));
+	fprintf(output, _("  SERVER_VERSION_NAME\n"
+					  "  SERVER_VERSION_NUM\n"
+					  "    server's version (in short string or numeric format)\n"));
+	fprintf(output, _("  SHOW_CONTEXT\n"
+					  "    controls display of message context fields [never, errors, always]\n"));
+	fprintf(output, _("  SINGLELINE\n"
+					  "    if set, end of line terminates SQL commands (same as -S option)\n"));
+	fprintf(output, _("  SINGLESTEP\n"
+					  "    single-step mode (same as -s option)\n"));
+	fprintf(output, _("  SQLSTATE\n"
+					  "    SQLSTATE of last query, or \"00000\" if no error\n"));
+	fprintf(output, _("  USER\n"
+					  "    the currently connected database user\n"));
+	fprintf(output, _("  VERBOSITY\n"
+					  "    controls verbosity of error reports [default, verbose, terse]\n"));
+	fprintf(output, _("  VERSION\n"
+					  "  VERSION_NAME\n"
+					  "  VERSION_NUM\n"
+					  "    psql's version (in verbose string, short string, or numeric format)\n"));
 
 	fprintf(output, _("\nDisplay settings:\n"));
 	fprintf(output, _("Usage:\n"));
 	fprintf(output, _("  psql --pset=NAME[=VALUE]\n  or \\pset NAME [VALUE] inside psql\n\n"));
 
-	fprintf(output, _("  border             border style (number)\n"));
-	fprintf(output, _("  columns            target width for the wrapped format\n"));
-	fprintf(output, _("  expanded (or x)    expanded output [on, off, auto]\n"));
-	fprintf(output, _("  fieldsep           field separator for unaligned output (default \"%s\")\n"), DEFAULT_FIELD_SEP);
-	fprintf(output, _("  fieldsep_zero      set field separator for unaligned output to zero byte\n"));
-	fprintf(output, _("  footer             enable or disable display of the table footer [on, off]\n"));
-	fprintf(output, _("  format             set output format [unaligned, aligned, wrapped, html, asciidoc, ...]\n"));
-	fprintf(output, _("  linestyle          set the border line drawing style [ascii, old-ascii, unicode]\n"));
-	fprintf(output, _("  null               set the string to be printed in place of a null value\n"));
-	fprintf(output, _("  numericlocale      enable or disable display of a locale-specific character to separate\n"
-					  "                     groups of digits [on, off]\n"));
-	fprintf(output, _("  pager              control when an external pager is used [yes, no, always]\n"));
-	fprintf(output, _("  recordsep          record (line) separator for unaligned output\n"));
-	fprintf(output, _("  recordsep_zero     set record separator for unaligned output to zero byte\n"));
-	fprintf(output, _("  tableattr (or T)   specify attributes for table tag in html format or proportional\n"
-					  "                     column widths for left-aligned data types in latex-longtable format\n"));
-	fprintf(output, _("  title              set the table title for any subsequently printed tables\n"));
-	fprintf(output, _("  tuples_only        if set, only actual table data is shown\n"));
+	fprintf(output, _("  border\n"
+					  "    border style (number)\n"));
+	fprintf(output, _("  columns\n"
+					  "    target width for the wrapped format\n"));
+	fprintf(output, _("  expanded (or x)\n"
+					  "    expanded output [on, off, auto]\n"));
+	fprintf(output, _("  fieldsep\n"
+					  "    field separator for unaligned output (default \"%s\")\n"),
+			DEFAULT_FIELD_SEP);
+	fprintf(output, _("  fieldsep_zero\n"
+					  "    set field separator for unaligned output to a zero byte\n"));
+	fprintf(output, _("  footer\n"
+					  "    enable or disable display of the table footer [on, off]\n"));
+	fprintf(output, _("  format\n"
+					  "    set output format [unaligned, aligned, wrapped, html, asciidoc, ...]\n"));
+	fprintf(output, _("  linestyle\n"
+					  "    set the border line drawing style [ascii, old-ascii, unicode]\n"));
+	fprintf(output, _("  null\n"
+					  "    set the string to be printed in place of a null value\n"));
+	fprintf(output, _("  numericlocale\n"
+					  "    enable display of a locale-specific character to separate groups of digits\n"));
+	fprintf(output, _("  pager\n"
+					  "    control when an external pager is used [yes, no, always]\n"));
+	fprintf(output, _("  recordsep\n"
+					  "    record (line) separator for unaligned output\n"));
+	fprintf(output, _("  recordsep_zero\n"
+					  "    set record separator for unaligned output to a zero byte\n"));
+	fprintf(output, _("  tableattr (or T)\n"
+					  "    specify attributes for table tag in html format, or proportional\n"
+					  "    column widths for left-aligned data types in latex-longtable format\n"));
+	fprintf(output, _("  title\n"
+					  "    set the table title for subsequently printed tables\n"));
+	fprintf(output, _("  tuples_only\n"
+					  "    if set, only actual table data is shown\n"));
 	fprintf(output, _("  unicode_border_linestyle\n"
 					  "  unicode_column_linestyle\n"
 					  "  unicode_header_linestyle\n"
-					  "                     set the style of Unicode line drawing [single, double]\n"));
+					  "    set the style of Unicode line drawing [single, double]\n"));
 
 	fprintf(output, _("\nEnvironment variables:\n"));
 	fprintf(output, _("Usage:\n"));
@@ -415,23 +467,36 @@ helpVariables(unsigned short int pager)
 	fprintf(output, _("  set NAME=VALUE\n  psql ...\n  or \\setenv NAME [VALUE] inside psql\n\n"));
 #endif
 
-	fprintf(output, _("  COLUMNS            number of columns for wrapped format\n"));
-	fprintf(output, _("  PAGER              name of external pager program\n"));
-	fprintf(output, _("  PGAPPNAME          same as the application_name connection parameter\n"));
-	fprintf(output, _("  PGDATABASE         same as the dbname connection parameter\n"));
-	fprintf(output, _("  PGHOST             same as the host connection parameter\n"));
-	fprintf(output, _("  PGPASSWORD         connection password (not recommended)\n"));
-	fprintf(output, _("  PGPASSFILE         password file name\n"));
-	fprintf(output, _("  PGPORT             same as the port connection parameter\n"));
-	fprintf(output, _("  PGUSER             same as the user connection parameter\n"));
+	fprintf(output, _("  COLUMNS\n"
+					  "    number of columns for wrapped format\n"));
+	fprintf(output, _("  PGAPPNAME\n"
+					  "    same as the application_name connection parameter\n"));
+	fprintf(output, _("  PGDATABASE\n"
+					  "    same as the dbname connection parameter\n"));
+	fprintf(output, _("  PGHOST\n"
+					  "    same as the host connection parameter\n"));
+	fprintf(output, _("  PGPASSWORD\n"
+					  "    connection password (not recommended)\n"));
+	fprintf(output, _("  PGPASSFILE\n"
+					  "    password file name\n"));
+	fprintf(output, _("  PGPORT\n"
+					  "    same as the port connection parameter\n"));
+	fprintf(output, _("  PGUSER\n"
+					  "    same as the user connection parameter\n"));
 	fprintf(output, _("  PSQL_EDITOR, EDITOR, VISUAL\n"
-					  "                     editor used by the \\e, \\ef, and \\ev commands\n"));
+					  "    editor used by the \\e, \\ef, and \\ev commands\n"));
 	fprintf(output, _("  PSQL_EDITOR_LINENUMBER_ARG\n"
-					  "                     how to specify a line number when invoking the editor\n"));
-	fprintf(output, _("  PSQL_HISTORY       alternative location for the command history file\n"));
-	fprintf(output, _("  PSQLRC             alternative location for the user's .psqlrc file\n"));
-	fprintf(output, _("  SHELL              shell used by the \\! command\n"));
-	fprintf(output, _("  TMPDIR             directory for temporary files\n"));
+					  "    how to specify a line number when invoking the editor\n"));
+	fprintf(output, _("  PSQL_HISTORY\n"
+					  "    alternative location for the command history file\n"));
+	fprintf(output, _("  PSQL_PAGER, PAGER\n"
+					  "    name of external pager program\n"));
+	fprintf(output, _("  PSQLRC\n"
+					  "    alternative location for the user's .psqlrc file\n"));
+	fprintf(output, _("  SHELL\n"
+					  "    shell used by the \\! command\n"));
+	fprintf(output, _("  TMPDIR\n"
+					  "    directory for temporary files\n"));
 
 	ClosePager(output);
 }
@@ -457,7 +522,6 @@ helpSQL(const char *topic, unsigned short int pager)
 		int			i;
 		int			j;
 
-		/* Find screen width to determine how many columns will fit */
 #ifdef TIOCGWINSZ
 		struct winsize screen_size;
 
@@ -495,53 +559,49 @@ helpSQL(const char *topic, unsigned short int pager)
 	else
 	{
 		int			i,
-					pass;
+					j,
+					x = 0;
+		bool		help_found = false;
 		FILE	   *output = NULL;
 		size_t		len,
-					wordlen,
-					j;
-		int			nl_count;
+					wordlen;
+		int			nl_count = 0;
 
 		/*
-		 * len is the amount of the input to compare to the help topic names.
 		 * We first try exact match, then first + second words, then first
 		 * word only.
 		 */
 		len = strlen(topic);
 
-		for (pass = 1; pass <= 3; pass++)
+		for (x = 1; x <= 3; x++)
 		{
-			if (pass > 1)		/* Nothing on first pass - try the opening
+			if (x > 1)			/* Nothing on first pass - try the opening
 								 * word(s) */
 			{
 				wordlen = j = 1;
-				while (j < len && topic[j++] != ' ')
+				while (topic[j] != ' ' && j++ < len)
 					wordlen++;
-				if (pass == 2 && j < len)
+				if (x == 2)
 				{
-					wordlen++;
-					while (j < len && topic[j++] != ' ')
+					j++;
+					while (topic[j] != ' ' && j++ <= len)
 						wordlen++;
 				}
-				if (wordlen >= len)
+				if (wordlen >= len) /* Don't try again if the same word */
 				{
-					/* Failed to shorten input, so try next pass if any */
-					continue;
+					if (!output)
+						output = PageOutput(nl_count, pager ? &(pset.popt.topt) : NULL);
+					break;
 				}
 				len = wordlen;
 			}
 
-			/*
-			 * Count newlines for pager.  This logic must agree with what the
-			 * following loop will do!
-			 */
-			nl_count = 0;
+			/* Count newlines for pager */
 			for (i = 0; QL_HELP[i].cmd; i++)
 			{
 				if (pg_strncasecmp(topic, QL_HELP[i].cmd, len) == 0 ||
 					strcmp(topic, "*") == 0)
 				{
-					/* magic constant here must match format below! */
 					nl_count += 5 + QL_HELP[i].nl_count;
 
 					/* If we have an exact match, exit.  Fixes \h SELECT */
@@ -549,9 +609,6 @@ helpSQL(const char *topic, unsigned short int pager)
 						break;
 				}
 			}
-			/* If no matches, don't open the output yet */
-			if (nl_count == 0)
-				continue;
 
 			if (!output)
 				output = PageOutput(nl_count, pager ? &(pset.popt.topt) : NULL);
@@ -565,31 +622,24 @@ helpSQL(const char *topic, unsigned short int pager)
 
 					initPQExpBuffer(&buffer);
 					QL_HELP[i].syntaxfunc(&buffer);
-					/* # of newlines in format must match constant above! */
+					help_found = true;
 					fprintf(output, _("Command:     %s\n"
 									  "Description: %s\n"
 									  "Syntax:\n%s\n\n"),
 							QL_HELP[i].cmd,
 							_(QL_HELP[i].help),
 							buffer.data);
-					termPQExpBuffer(&buffer);
-
 					/* If we have an exact match, exit.  Fixes \h SELECT */
 					if (pg_strcasecmp(topic, QL_HELP[i].cmd) == 0)
 						break;
 				}
 			}
-			break;
+			if (help_found)		/* Don't keep trying if we got a match */
+				break;
 		}
 
-		/* If we never found anything, report that */
-		if (!output)
-		{
-			output = PageOutput(2, pager ? &(pset.popt.topt) : NULL);
-			fprintf(output, _("No help available for \"%s\".\n"
-							  "Try \\h with no arguments to see available help.\n"),
-					topic);
-		}
+		if (!help_found)
+			fprintf(output, _("No help available for \"%s\".\nTry \\h with no arguments to see available help.\n"), topic);
 
 		ClosePager(output);
 	}
@@ -603,7 +653,7 @@ print_copyright(void)
 	puts(
 		 "PostgreSQL Database Management System\n"
 		 "(formerly known as Postgres, then as Postgres95)\n\n"
-		 "Portions Copyright (c) 1996-2017, PostgreSQL Global Development Group\n\n"
+		 "Portions Copyright (c) 1996-2018, PostgreSQL Global Development Group\n\n"
 		 "Portions Copyright (c) 1994, The Regents of the University of California\n\n"
 		 "Permission to use, copy, modify, and distribute this software and its\n"
 		 "documentation for any purpose, without fee, and without a written agreement\n"
