@@ -1,17 +1,19 @@
 /*
  * psql - the PostgreSQL interactive terminal
  *
- * Copyright (c) 2000-2014, PostgreSQL Global Development Group
+ * Copyright (c) 2000-2018, PostgreSQL Global Development Group
  *
  * src/bin/psql/help.h
  */
 #ifndef HELP_H
 #define HELP_H
 
-void		usage(void);
+void		usage(unsigned short int pager);
 
 void		slashUsage(unsigned short int pager);
 
+void		helpVariables(unsigned short int pager);
+
 void		helpSQL(const char *topic, unsigned short int pager);
 
 void		print_copyright(void);
