@@ -29,10 +29,9 @@ PostgresNode - class representing PostgreSQL server instance
   # as well as the psql exit code. Pass some extra psql
   # options. If there's an error from psql raise an exception.
   my ($stdout, $stderr, $timed_out);
-  my $cmdret = $node->psql('postgres', 'SELECT pg_sleep(600)',
+  my $cmdret = $node->psql('postgres', 'SELECT pg_sleep(60)',
 	  stdout => \$stdout, stderr => \$stderr,
-	  timeout => $TestLib::timeout_default,
-	  timed_out => \$timed_out,
+	  timeout => 30, timed_out => \$timed_out,
 	  extra_params => ['--single-transaction'],
 	  on_error_die => 1)
   print "Sleep timed out" if $timed_out;
@@ -64,9 +63,6 @@ PostgresNode - class representing PostgreSQL server instance
   # Stop the server
   $node->stop('fast');
 
-  # Find a free, unprivileged TCP port to bind some other service to
-  my $port = get_free_port();
-
 =head1 DESCRIPTION
 
 PostgresNode contains a set of routines able to work on a PostgreSQL node,
@@ -90,9 +86,9 @@ use Carp;
 use Config;
 use Cwd;
 use Exporter 'import';
-use Fcntl qw(:mode :flock :seek :DEFAULT);
+use Fcntl qw(:mode);
 use File::Basename;
-use File::Path qw(rmtree mkpath);
+use File::Path qw(rmtree);
 use File::Spec;
 use File::stat qw(stat);
 use File::Temp ();
@@ -106,16 +102,12 @@ use Scalar::Util qw(blessed);
 
 our @EXPORT = qw(
   get_new_node
-  get_free_port
 );
 
-our ($use_tcp, $test_localhost, $test_pghost, $last_host_assigned,
-	$last_port_assigned, @all_nodes, $died, $portdir);
+our ($test_localhost, $test_pghost, $last_port_assigned, @all_nodes, $died);
 
-# list of file reservations made by get_free_port
-my @port_reservation_files;
+# Windows path to virtual file system root
 
-# For backward compatibility only.
 our $vfs_path = '';
 if ($Config{osname} eq 'msys')
 {
@@ -126,31 +118,16 @@ if ($Config{osname} eq 'msys')
 INIT
 {
 
-	# Set PGHOST for backward compatibility.  This doesn't work for own_host
-	# nodes, so prefer to not rely on this when writing new tests.
-	$use_tcp            = $TestLib::windows_os;
-	$test_localhost     = "127.0.0.1";
-	$last_host_assigned = 1;
-	$test_pghost        = $use_tcp ? $test_localhost : TestLib::tempdir_short;
-	$ENV{PGHOST}        = $test_pghost;
-	$ENV{PGDATABASE}    = 'postgres';
+	# PGHOST is set once and for all through a single series of tests when
+	# this module is loaded.
+	$test_localhost = "127.0.0.1";
+	$test_pghost =
+	  $TestLib::windows_os ? $test_localhost : TestLib::tempdir_short;
+	$ENV{PGHOST}     = $test_pghost;
+	$ENV{PGDATABASE} = 'postgres';
 
 	# Tracking of last port value assigned to accelerate free port lookup.
 	$last_port_assigned = int(rand() * 16384) + 49152;
-
-	# Set the port lock directory
-
-	# If we're told to use a directory (e.g. from a buildfarm client)
-	# explicitly, use that
-	$portdir = $ENV{PG_TEST_PORT_DIR};
-	# Otherwise, try to use a directory at the top of the build tree
-	# or as a last resort use the tmp_check directory
-	my $build_dir = $ENV{top_builddir}
-	  || $TestLib::tmp_check ;
-	$portdir ||= "$build_dir/portlock";
-	$portdir =~ s!\\!/!g;
-	# Make sure the directory exists
-	mkpath($portdir) unless -d $portdir;
 }
 
 =pod
@@ -171,17 +148,6 @@ of finding port numbers, registering instances for cleanup, etc.
 sub new
 {
 	my ($class, $name, $pghost, $pgport) = @_;
-
-	# Use release 15+ semantics when the arguments look like (node_name,
-	# %params).  We can't use $class to decide, because get_new_node() passes
-	# a v14- argument list regardless of the class.  $class might be an
-	# out-of-core subclass.  $class->isa('PostgresNode') returns true even for
-	# descendants of PostgreSQL::Test::Cluster, so it doesn't help.
-	return $class->get_new_node(@_[ 1 .. $#_ ])
-	  if !$pghost
-	  or !$pgport
-	  or $pghost =~ /^[a-zA-Z0-9_]$/;
-
 	my $testname = basename($0);
 	$testname =~ s/\.[^.]+$//;
 	my $self = {
@@ -189,9 +155,7 @@ sub new
 		_host    => $pghost,
 		_basedir => "$TestLib::tmp_check/t_${testname}_${name}_data",
 		_name    => $name,
-		_logfile_generation => 0,
-		_logfile_base       => "$TestLib::log_path/${testname}_${name}",
-		_logfile            => "$TestLib::log_path/${testname}_${name}.log"
+		_logfile => "$TestLib::log_path/${testname}_${name}.log"
 	};
 
 	bless $self, $class;
@@ -473,8 +437,7 @@ sub init
 
 	TestLib::system_or_bail('initdb', '-D', $pgdata, '-A', 'trust', '-N',
 		@{ $params{extra} });
-	TestLib::system_or_bail($ENV{PG_REGRESS}, '--config-auth', $pgdata,
-		@{ $params{auth_extra} });
+	TestLib::system_or_bail($ENV{PG_REGRESS}, '--config-auth', $pgdata);
 
 	open my $conf, '>>', "$pgdata/postgresql.conf";
 	print $conf "\n# Added by PostgresNode.pm\n";
@@ -484,17 +447,7 @@ sub init
 	print $conf "log_statement = all\n";
 	print $conf "log_replication_commands = on\n";
 	print $conf "wal_retrieve_retry_interval = '500ms'\n";
-
-	# If a setting tends to affect whether tests pass or fail, print it after
-	# TEMP_CONFIG.  Otherwise, print it before TEMP_CONFIG, thereby permitting
-	# overrides.  Settings that merely improve performance or ease debugging
-	# belong before TEMP_CONFIG.
-	print $conf TestLib::slurp_file($ENV{TEMP_CONFIG})
-	  if defined $ENV{TEMP_CONFIG};
-
-	# XXX Neutralize any stats_temp_directory in TEMP_CONFIG.  Nodes running
-	# concurrently must not share a stats_temp_directory.
-	print $conf "stats_temp_directory = 'pg_stat_tmp'\n";
+	print $conf "port = $port\n";
 
 	if ($params{allows_streaming})
 	{
@@ -506,15 +459,13 @@ sub init
 		{
 			print $conf "wal_level = replica\n";
 		}
-		print $conf "max_wal_senders = 10\n";
-		print $conf "max_replication_slots = 10\n";
+		print $conf "max_wal_senders = 5\n";
+		print $conf "max_replication_slots = 5\n";
+		print $conf "max_wal_size = 128MB\n";
+		print $conf "shared_buffers = 1MB\n";
 		print $conf "wal_log_hints = on\n";
 		print $conf "hot_standby = on\n";
-		# conservative settings to ensure we can run multiple postmasters:
-		print $conf "shared_buffers = 1MB\n";
-		print $conf "max_connections = 20\n";
-		# limit disk space consumption, too:
-		print $conf "max_wal_size = 128MB\n";
+		print $conf "max_connections = 10\n";
 	}
 	else
 	{
@@ -522,10 +473,8 @@ sub init
 		print $conf "max_wal_senders = 0\n";
 	}
 
-	print $conf "port = $port\n";
-	if ($use_tcp)
+	if ($TestLib::windows_os)
 	{
-		print $conf "unix_socket_directories = ''\n";
 		print $conf "listen_addresses = '$host'\n";
 	}
 	else
@@ -575,11 +524,8 @@ sub append_conf
 =item $node->backup(backup_name)
 
 Create a hot backup with B<pg_basebackup> in subdirectory B<backup_name> of
-B<< $node->backup_dir >>, including the WAL.
-
-By default, WAL files are fetched at the end of the backup, not streamed.
-You can adjust that and other things by passing an array of additional
-B<pg_basebackup> command line options in the keyword parameter backup_options.
+B<< $node->backup_dir >>, including the WAL. WAL files
+fetched at the end of the backup, not streamed.
 
 You'll have to configure a suitable B<max_wal_senders> on the
 target server since it isn't done by default.
@@ -588,16 +534,14 @@ target server since it isn't done by default.
 
 sub backup
 {
-	my ($self, $backup_name, %params) = @_;
+	my ($self, $backup_name) = @_;
 	my $backup_path = $self->backup_dir . '/' . $backup_name;
+	my $port        = $self->port;
 	my $name        = $self->name;
 
 	print "# Taking pg_basebackup $backup_name from node \"$name\"\n";
-	TestLib::system_or_bail(
-		'pg_basebackup', '-D', $backup_path, '-h',
-		$self->host,     '-p', $self->port,  '--checkpoint',
-		'fast',          '--no-sync',
-		@{ $params{backup_options} });
+	TestLib::system_or_bail('pg_basebackup', '-D', $backup_path, '-p', $port,
+		'--no-sync');
 	print "# Backup finished\n";
 	return;
 }
@@ -709,7 +653,6 @@ sub init_from_backup
 {
 	my ($self, $root_node, $backup_name, %params) = @_;
 	my $backup_path = $root_node->backup_dir . '/' . $backup_name;
-	my $host        = $self->host;
 	my $port        = $self->port;
 	my $node_name   = $self->name;
 	my $root_name   = $root_node->name;
@@ -736,15 +679,6 @@ sub init_from_backup
 		qq(
 port = $port
 ));
-	if ($use_tcp)
-	{
-		$self->append_conf('postgresql.conf', "listen_addresses = '$host'");
-	}
-	else
-	{
-		$self->append_conf('postgresql.conf',
-			"unix_socket_directories = '$host'");
-	}
 	$self->enable_streaming($root_node) if $params{has_streaming};
 	$self->enable_restoring($root_node) if $params{has_restoring};
 	return;
@@ -752,45 +686,17 @@ port = $port
 
 =pod
 
-=item $node->rotate_logfile()
-
-Switch to a new PostgreSQL log file.  This does not alter any running
-PostgreSQL process.  Subsequent method calls, including pg_ctl invocations,
-will use the new name.  Return the new name.
-
-=cut
-
-sub rotate_logfile
-{
-	my ($self) = @_;
-	$self->{_logfile} = sprintf('%s_%d.log',
-		$self->{_logfile_base},
-		++$self->{_logfile_generation});
-	return $self->{_logfile};
-}
-
-=pod
-
-=item $node->start(%params) => success_or_failure
+=item $node->start()
 
 Wrapper for pg_ctl start
 
 Start the node and wait until it is ready to accept connections.
 
-=over
-
-=item fail_ok => 1
-
-By default, failure terminates the entire F<prove> invocation.  If given,
-instead return a true or false value to indicate success or failure.
-
-=back
-
 =cut
 
 sub start
 {
-	my ($self, %params) = @_;
+	my ($self) = @_;
 	my $port   = $self->port;
 	my $pgdata = $self->data_dir;
 	my $name   = $self->name;
@@ -803,39 +709,10 @@ sub start
 	{
 		print "# pg_ctl start failed; logfile:\n";
 		print TestLib::slurp_file($self->logfile);
-
-		# pg_ctl could have timed out, so check to see if there's a pid file;
-		# otherwise our END block will fail to shut down the new postmaster.
-		$self->_update_pid(-1);
-
-		BAIL_OUT("pg_ctl start failed") unless $params{fail_ok};
-		return 0;
+		BAIL_OUT("pg_ctl start failed");
 	}
 
 	$self->_update_pid(1);
-	return 1;
-}
-
-=pod
-
-=item $node->kill9()
-
-Send SIGKILL (signal 9) to the postmaster.
-
-Note: if the node is already known stopped, this does nothing.
-However, if we think it's running and it's not, it's important for
-this to fail.  Otherwise, tests might fail to detect server crashes.
-
-=cut
-
-sub kill9
-{
-	my ($self) = @_;
-	my $name = $self->name;
-	return unless defined $self->{_pid};
-	print "### Killing node \"$name\" using signal 9\n";
-	kill(9, $self->{_pid});
-	$self->{_pid} = undef;
 	return;
 }
 
@@ -849,37 +726,20 @@ Note: if the node is already known stopped, this does nothing.
 However, if we think it's running and it's not, it's important for
 this to fail.  Otherwise, tests might fail to detect server crashes.
 
-With optional extra param fail_ok => 1, returns 0 for failure
-instead of bailing out.
-
 =cut
 
 sub stop
 {
-	my ($self, $mode, %params) = @_;
+	my ($self, $mode) = @_;
+	my $port   = $self->port;
 	my $pgdata = $self->data_dir;
 	my $name   = $self->name;
-	my $ret;
 	$mode = 'fast' unless defined $mode;
-	return 1 unless defined $self->{_pid};
-
+	return unless defined $self->{_pid};
 	print "### Stopping node \"$name\" using mode $mode\n";
-	$ret = TestLib::system_log('pg_ctl', '-D', $pgdata,
-		'-m', $mode, 'stop');
-
-	if ($ret != 0)
-	{
-		print "# pg_ctl stop failed: $ret\n";
-
-		# Check to see if we still have a postmaster or not.
-		$self->_update_pid(-1);
-
-		BAIL_OUT("pg_ctl stop failed") unless $params{fail_ok};
-		return 0;
-	}
-
+	TestLib::system_or_bail('pg_ctl', '-D', $pgdata, '-m', $mode, 'stop');
 	$self->_update_pid(0);
-	return 1;
+	return;
 }
 
 =pod
@@ -964,7 +824,7 @@ standby_mode=on
 sub enable_restoring
 {
 	my ($self, $root_node) = @_;
-	my $path = $root_node->archive_dir;
+	my $path = $vfs_path . $root_node->archive_dir;
 	my $name = $self->name;
 
 	print "### Enabling WAL restore for node \"$name\"\n";
@@ -993,7 +853,7 @@ standby_mode = on
 sub enable_archiving
 {
 	my ($self) = @_;
-	my $path   = $self->archive_dir;
+	my $path   = $vfs_path . $self->archive_dir;
 	my $name   = $self->name;
 
 	print "### Enabling WAL archiving for node \"$name\"\n";
@@ -1019,10 +879,7 @@ archive_command = '$copy_command'
 	return;
 }
 
-# Internal method to update $self->{_pid}
-# $is_running = 1: pid file should be there
-# $is_running = 0: pid file should NOT be there
-# $is_running = -1: we aren't sure
+# Internal method
 sub _update_pid
 {
 	my ($self, $is_running) = @_;
@@ -1033,22 +890,11 @@ sub _update_pid
 	if (open my $pidfile, '<', $self->data_dir . "/postmaster.pid")
 	{
 		chomp($self->{_pid} = <$pidfile>);
+		print "# Postmaster PID for node \"$name\" is $self->{_pid}\n";
 		close $pidfile;
 
-		# If we aren't sure what to expect, validate the PID using kill().
-		# This protects against stale PID files left by crashed postmasters.
-		if ($is_running == -1 && kill(0, $self->{_pid}) == 0)
-		{
-			print
-			  "# Stale postmaster.pid file for node \"$name\": PID $self->{_pid} no longer exists\n";
-			$self->{_pid} = undef;
-			return;
-		}
-
-		print "# Postmaster PID for node \"$name\" is $self->{_pid}\n";
-
 		# If we found a pidfile when there shouldn't be one, complain.
-		BAIL_OUT("postmaster.pid unexpectedly present") if $is_running == 0;
+		BAIL_OUT("postmaster.pid unexpectedly present") unless $is_running;
 		return;
 	}
 
@@ -1056,13 +902,13 @@ sub _update_pid
 	print "# No postmaster PID for node \"$name\"\n";
 
 	# Complain if we expected to find a pidfile.
-	BAIL_OUT("postmaster.pid unexpectedly not present") if $is_running == 1;
+	BAIL_OUT("postmaster.pid unexpectedly not present") if $is_running;
 	return;
 }
 
 =pod
 
-=item PostgresNode->get_new_node(node_name, %params)
+=item PostgresNode->get_new_node(node_name)
 
 Build a new object of class C<PostgresNode> (or of a subclass, if you have
 one), assigning a free port number.  Remembers the node, to prevent its port
@@ -1071,22 +917,6 @@ shut down when the test script exits.
 
 You should generally use this instead of C<PostgresNode::new(...)>.
 
-=over
-
-=item port => [1,65535]
-
-By default, this function assigns a port number to each node.  Specify this to
-force a particular port number.  The caller is responsible for evaluating
-potential conflicts and privilege requirements.
-
-=item own_host => 1
-
-By default, all nodes use the same PGHOST value.  If specified, generate a
-PGHOST specific to this node.  This allows multiple nodes to use the same
-port.
-
-=back
-
 For backwards compatibility, it is also exported as a standalone function,
 which can only create objects of class C<PostgresNode>.
 
@@ -1095,68 +925,8 @@ which can only create objects of class C<PostgresNode>.
 sub get_new_node
 {
 	my $class = 'PostgresNode';
-	$class = shift if scalar(@_) % 2 != 1;
-	my ($name, %params) = @_;
-
-	# Select a port.
-	my $port;
-	if (defined $params{port})
-	{
-		$port = $params{port};
-	}
-	else
-	{
-		# When selecting a port, we look for an unassigned TCP port number,
-		# even if we intend to use only Unix-domain sockets.  This is clearly
-		# necessary on $use_tcp (Windows) configurations, and it seems like a
-		# good idea on Unixen as well.
-		$port = get_free_port();
-	}
-
-	# Select a host.
-	my $host = $test_pghost;
-	if ($params{own_host})
-	{
-		if ($use_tcp)
-		{
-			$last_host_assigned++;
-			$last_host_assigned > 254 and BAIL_OUT("too many own_host nodes");
-			$host = '127.0.0.' . $last_host_assigned;
-		}
-		else
-		{
-			$host = "$test_pghost/$name"; # Assume $name =~ /^[-_a-zA-Z0-9]+$/
-			mkdir $host;
-		}
-	}
-
-	# Lock port number found by creating a new node
-	my $node = $class->new($name, $host, $port);
-
-	# Add node to list of nodes
-	push(@all_nodes, $node);
-
-	return $node;
-}
-
-=pod
-
-=item get_free_port()
-
-Locate an unprivileged (high) TCP port that's not currently bound to
-anything.  This is used by get_new_node, and is also exported for use
-by test cases that need to start other, non-Postgres servers.
-
-Ports assigned to existing PostgresNode objects are automatically
-excluded, even if those servers are not currently running.
-
-The port number is reserved so that other concurrent test programs will not
-try to use the same port.
-
-=cut
-
-sub get_free_port
-{
+	$class = shift if 1 < scalar @_;
+	my $name  = shift;
 	my $found = 0;
 	my $port  = $last_port_assigned;
 
@@ -1176,94 +946,54 @@ sub get_free_port
 		}
 
 		# Check to see if anything else is listening on this TCP port.
-		# Seek a port available for all possible listen_addresses values,
-		# so callers can harness this port for the widest range of purposes.
-		# The 0.0.0.0 test achieves that for MSYS, which automatically sets
-		# SO_EXCLUSIVEADDRUSE.  Testing 0.0.0.0 is insufficient for Windows
-		# native Perl (https://stackoverflow.com/a/14388707), so we also
-		# have to test individual addresses.  Doing that for 127.0.0/24
-		# addresses other than 127.0.0.1 might fail with EADDRNOTAVAIL on
-		# non-Linux, non-Windows kernels.
-		#
-		# Thus, 0.0.0.0 and individual 127.0.0/24 addresses are tested
-		# only on Windows and only when TCP usage is requested.
+		# This is *necessary* on Windows, and seems like a good idea
+		# on Unixen as well, even though we don't ask the postmaster
+		# to open a TCP port on Unix.
 		if ($found == 1)
 		{
-			foreach my $addr (qw(127.0.0.1),
-               ($use_tcp && $TestLib::windows_os)
-               ? qw(127.0.0.2 127.0.0.3 0.0.0.0)
-               : ())
-			{
-				if (!can_bind($addr, $port))
-				{
-					$found = 0;
-					last;
-				}
-			}
-			$found = _reserve_port($port) if $found;
+			my $iaddr = inet_aton($test_localhost);
+			my $paddr = sockaddr_in($port, $iaddr);
+			my $proto = getprotobyname("tcp");
+
+			socket(SOCK, PF_INET, SOCK_STREAM, $proto)
+			  or die "socket failed: $!";
+
+			# As in postmaster, don't use SO_REUSEADDR on Windows
+			setsockopt(SOCK, SOL_SOCKET, SO_REUSEADDR, pack("l", 1))
+			  unless $TestLib::windows_os;
+			(bind(SOCK, $paddr) && listen(SOCK, SOMAXCONN))
+			  or $found = 0;
+			close(SOCK);
 		}
 	}
 
-	print "# Found port $port\n";
+	print "# Found free port $port\n";
 
-	# Update port for next time
+	# Lock port number found by creating a new node
+	my $node = $class->new($name, $test_pghost, $port);
+
+	# Add node to list of nodes
+	push(@all_nodes, $node);
+
+	# And update port for next time
 	$last_port_assigned = $port;
 
-	return $port;
+	return $node;
 }
 
-# Internal routine to check whether a host:port is available to bind
-sub can_bind
-{
-	my ($host, $port) = @_;
-	my $iaddr = inet_aton($host);
-	my $paddr = sockaddr_in($port, $iaddr);
-	my $proto = getprotobyname("tcp");
-
-	socket(SOCK, PF_INET, SOCK_STREAM, $proto)
-	  or die "socket failed: $!";
-
-	# As in postmaster, don't use SO_REUSEADDR on Windows
-	setsockopt(SOCK, SOL_SOCKET, SO_REUSEADDR, pack("l", 1))
-	  unless $TestLib::windows_os;
-	my $ret = bind(SOCK, $paddr) && listen(SOCK, SOMAXCONN);
-	close(SOCK);
-	return $ret;
-}
-
-# Internal routine to reserve a port number
-# Returns 1 if successful, 0 if port is already reserved.
-sub _reserve_port
-{
-	my $port = shift;
-	# open in rw mode so we don't have to reopen it and lose the lock
-	my $filename = "$portdir/$port.rsv";
-	sysopen(my $portfile, $filename, O_RDWR|O_CREAT)
-	  || die "opening port file $filename: $!";
-	# take an exclusive lock to avoid concurrent access
-	flock($portfile, LOCK_EX) || die "locking port file $filename: $!";
-	# see if someone else has or had a reservation of this port
-	my $pid = <$portfile> || "0";
-	chomp $pid;
-	if ($pid +0 > 0)
+# Retain the errno on die() if set, else assume a generic errno of 1.
+# This will instruct the END handler on how to handle artifacts left
+# behind from tests.
+$SIG{__DIE__} = sub {
+	if ($!)
 	{
-		if (kill 0, $pid)
-		{
-			# process exists and is owned by us, so we can't reserve this port
-			flock($portfile, LOCK_UN);
-			close($portfile);
-			return 0;
-		}
+		$died = $!;
 	}
-	# All good, go ahead and reserve the port
-	seek($portfile, 0, SEEK_SET);
-	# print the pid with a fixed width so we don't leave any trailing junk
-	print $portfile sprintf("%10d\n",$$);
-	flock($portfile, LOCK_UN);
-	close($portfile);
-	push(@port_reservation_files, $filename);
-	return 1;
-}
+	else
+	{
+		$died = 1;
+	}
+};
 
 # Automatically shut down any still-running nodes when the test script exits.
 # Note that this just stops the postmasters (in the same order the nodes were
@@ -1283,11 +1013,10 @@ END
 		next if defined $ENV{'PG_TEST_NOCLEAN'};
 
 		# clean basedir on clean test invocation
-		$node->clean_node if $exit_code == 0 && TestLib::all_tests_passing();
+		$node->clean_node
+		  if TestLib::all_tests_passing() && !defined $died && !$exit_code;
 	}
 
-	unlink @port_reservation_files;
-
 	$? = $exit_code;
 }
 
@@ -1357,6 +1086,7 @@ sub safe_psql
 		print "\n#### End standard error\n";
 	}
 
+	$stdout =~ s/\r//g if $TestLib::windows_os;
 	return $stdout;
 }
 
@@ -1427,10 +1157,9 @@ If given, it must be an array reference containing additional parameters to B<ps
 e.g.
 
 	my ($stdout, $stderr, $timed_out);
-	my $cmdret = $node->psql('postgres', 'SELECT pg_sleep(600)',
+	my $cmdret = $node->psql('postgres', 'SELECT pg_sleep(60)',
 		stdout => \$stdout, stderr => \$stderr,
-		timeout => $TestLib::timeout_default,
-		timed_out => \$timed_out,
+		timeout => 30, timed_out => \$timed_out,
 		extra_params => ['--single-transaction'])
 
 will set $cmdret to undef and $timed_out to a true value.
@@ -1536,11 +1265,13 @@ sub psql
 	if (defined $$stdout)
 	{
 		chomp $$stdout;
+		$$stdout =~ s/\r//g if $TestLib::windows_os;
 	}
 
 	if (defined $$stderr)
 	{
 		chomp $$stderr;
+		$$stderr =~ s/\r//g if $TestLib::windows_os;
 	}
 
 	# See http://perldoc.perl.org/perlvar.html#%24CHILD_ERROR
@@ -1578,238 +1309,12 @@ sub psql
 
 =pod
 
-=item $node->background_psql($dbname, \$stdin, \$stdout, $timer, %params) => harness
-
-Invoke B<psql> on B<$dbname> and return an IPC::Run harness object, which the
-caller may use to send input to B<psql>.  The process's stdin is sourced from
-the $stdin scalar reference, and its stdout and stderr go to the $stdout
-scalar reference.  This allows the caller to act on other parts of the system
-while idling this backend.
-
-The specified timer object is attached to the harness, as well.  It's caller's
-responsibility to set the timeout length (usually
-$TestLib::timeout_default), and to restart the timer after
-each command if the timeout is per-command.
-
-psql is invoked in tuples-only unaligned mode with reading of B<.psqlrc>
-disabled.  That may be overridden by passing extra psql parameters.
-
-Dies on failure to invoke psql, or if psql fails to connect.  Errors occurring
-later are the caller's problem.  psql runs with on_error_stop by default so
-that it will stop running sql and return 3 if passed SQL results in an error.
-
-Be sure to "finish" the harness when done with it.
-
-=over
-
-=item on_error_stop => 1
-
-By default, the B<psql> method invokes the B<psql> program with ON_ERROR_STOP=1
-set, so SQL execution is stopped at the first error and exit code 3 is
-returned.  Set B<on_error_stop> to 0 to ignore errors instead.
-
-=item replication => B<value>
-
-If set, add B<replication=value> to the conninfo string.
-Passing the literal value C<database> results in a logical replication
-connection.
-
-=item extra_params => ['--single-transaction']
-
-If given, it must be an array reference containing additional parameters to B<psql>.
-
-=back
-
-=cut
-
-sub background_psql
-{
-	my ($self, $dbname, $stdin, $stdout, $timer, %params) = @_;
-
-	local $ENV{PGHOST} = $self->host;
-	local $ENV{PGPORT} = $self->port;
-
-	my $replication = $params{replication};
-
-	my @psql_params = (
-		'psql',
-		'-XAtq',
-		'-d',
-		$self->connstr($dbname)
-		  . (defined $replication ? " replication=$replication" : ""),
-		'-f',
-		'-');
-
-	$params{on_error_stop} = 1 unless defined $params{on_error_stop};
-
-	push @psql_params, '-v', 'ON_ERROR_STOP=1' if $params{on_error_stop};
-	push @psql_params, @{ $params{extra_params} }
-	  if defined $params{extra_params};
-
-	# Ensure there is no data waiting to be sent:
-	$$stdin = "" if ref($stdin);
-	# IPC::Run would otherwise append to existing contents:
-	$$stdout = "" if ref($stdout);
-
-	my $harness = IPC::Run::start \@psql_params,
-	  '<', $stdin, '>', $stdout, $timer;
-
-	# Request some output, and pump until we see it.  This means that psql
-	# connection failures are caught here, relieving callers of the need to
-	# handle those.  (Right now, we have no particularly good handling for
-	# errors anyway, but that might be added later.)
-	my $banner = "background_psql: ready";
-	$$stdin = "\\echo $banner\n";
-	pump $harness until $$stdout =~ /$banner/ || $timer->is_expired;
-
-	die "psql startup timed out" if $timer->is_expired;
-
-	return $harness;
-}
-
-# Common sub of pgbench-invoking interfaces.  Makes any requested script files
-# and returns pgbench command-line options causing use of those files.
-sub _pgbench_make_files
-{
-	my ($self, $files) = @_;
-	my @file_opts;
-
-	if (defined $files)
-	{
-
-		# note: files are ordered for determinism
-		for my $fn (sort keys %$files)
-		{
-			my $filename = $self->basedir . '/' . $fn;
-			push @file_opts, '-f', $filename;
-
-			# cleanup file weight
-			$filename =~ s/\@\d+$//;
-
-			#push @filenames, $filename;
-			# filenames are expected to be unique on a test
-			if (-e $filename)
-			{
-				ok(0, "$filename must not already exist");
-				unlink $filename or die "cannot unlink $filename: $!";
-			}
-			TestLib::append_to_file($filename, $$files{$fn});
-		}
-	}
-
-	return @file_opts;
-}
-
-=pod
-
-=item $node->pgbench($opts, $stat, $out, $err, $name, $files, @args)
-
-Invoke B<pgbench>, with parameters and files.
-
-=over
-
-=item $opts
-
-Options as a string to be split on spaces.
-
-=item $stat
-
-Expected exit status.
-
-=item $out
-
-Reference to a regexp list that must match stdout.
-
-=item $err
-
-Reference to a regexp list that must match stderr.
-
-=item $name
-
-Name of test for error messages.
-
-=item $files
-
-Reference to filename/contents dictionary.
-
-=item @args
-
-Further raw options or arguments.
-
-=back
-
-=cut
-
-sub pgbench
-{
-	local $Test::Builder::Level = $Test::Builder::Level + 1;
-
-	my ($self, $opts, $stat, $out, $err, $name, $files, @args) = @_;
-	my @cmd = (
-		'pgbench',
-		split(/\s+/, $opts),
-		$self->_pgbench_make_files($files), @args);
-
-	$self->command_checks_all(\@cmd, $stat, $out, $err, $name);
-}
-
-=pod
-
-=item $node->background_pgbench($opts, $files, \$stdout, $timer) => harness
-
-Invoke B<pgbench> and return an IPC::Run harness object.  The process's stdin
-is empty, and its stdout and stderr go to the $stdout scalar reference.  This
-allows the caller to act on other parts of the system while B<pgbench> is
-running.  Errors from B<pgbench> are the caller's problem.
-
-The specified timer object is attached to the harness, as well.  It's caller's
-responsibility to select the timeout length, and to restart the timer after
-each command if the timeout is per-command.
-
-Be sure to "finish" the harness when done with it.
-
-=over
-
-=item $opts
-
-Options as a string to be split on spaces.
-
-=item $files
-
-Reference to filename/contents dictionary.
-
-=back
-
-=cut
-
-sub background_pgbench
-{
-	my ($self, $opts, $files, $stdout, $timer) = @_;
-
-	my @cmd =
-	  ('pgbench', split(/\s+/, $opts), $self->_pgbench_make_files($files));
-
-	local $ENV{PGHOST} = $self->host;
-	local $ENV{PGPORT} = $self->port;
-
-	my $stdin = "";
-	# IPC::Run would otherwise append to existing contents:
-	$$stdout = "" if ref($stdout);
-
-	my $harness = IPC::Run::start \@cmd, '<', \$stdin, '>', $stdout, '2>&1',
-	  $timer;
-
-	return $harness;
-}
-
-=pod
-
 =item $node->poll_query_until($dbname, $query [, $expected ])
 
 Run B<$query> repeatedly, until it returns the B<$expected> result
 ('t', or SQL boolean true, by default).
 Continues polling if B<psql> returns an error result.
-Times out after $TestLib::timeout_default seconds.
+Times out after 180 seconds.
 Returns 1 if successful, 0 if timed out.
 
 =cut
@@ -1820,20 +1325,19 @@ sub poll_query_until
 
 	$expected = 't' unless defined($expected);    # default value
 
-	my $cmd = [ 'psql', '-XAt', '-d', $self->connstr($dbname) ];
+	my $cmd = [ 'psql', '-XAt', '-c', $query, '-d', $self->connstr($dbname) ];
 	my ($stdout, $stderr);
-	my $max_attempts = 10 * $TestLib::timeout_default;
+	my $max_attempts = 180 * 10;
 	my $attempts     = 0;
 
 	while ($attempts < $max_attempts)
 	{
-		my $result = IPC::Run::run $cmd, '<', \$query,
-		  '>', \$stdout, '2>', \$stderr;
+		my $result = IPC::Run::run $cmd, '>', \$stdout, '2>', \$stderr;
 
 		chomp($stdout);
-		chomp($stderr);
+		$stdout =~ s/\r//g if $TestLib::windows_os;
 
-		if ($stdout eq $expected && $stderr eq '')
+		if ($stdout eq $expected)
 		{
 			return 1;
 		}
@@ -1844,8 +1348,10 @@ sub poll_query_until
 		$attempts++;
 	}
 
-	# Give up. Print the output from the last attempt, hopefully that's useful
-	# for debugging.
+	# The query result didn't change in 180 seconds. Give up. Print the
+	# output from the last attempt, hopefully that's useful for debugging.
+	chomp($stderr);
+	$stderr =~ s/\r//g if $TestLib::windows_os;
 	diag qq(poll_query_until timed out executing this query:
 $query
 expecting this output:
@@ -1861,8 +1367,9 @@ $stderr);
 
 =item $node->command_ok(...)
 
-Runs a shell command like TestLib::command_ok, but with PGHOST and PGPORT set
-so that the command will default to connecting to this PostgresNode.
+Runs a shell command like TestLib::command_ok, but with PGPORT
+set so that the command will default to connecting to this
+PostgresNode.
 
 =cut
 
@@ -1870,7 +1377,6 @@ sub command_ok
 {
 	my $self = shift;
 
-	local $ENV{PGHOST} = $self->host;
 	local $ENV{PGPORT} = $self->port;
 
 	TestLib::command_ok(@_);
@@ -1881,7 +1387,7 @@ sub command_ok
 
 =item $node->command_fails(...)
 
-TestLib::command_fails with our connection parameters. See command_ok(...)
+TestLib::command_fails with our PGPORT. See command_ok(...)
 
 =cut
 
@@ -1889,7 +1395,6 @@ sub command_fails
 {
 	my $self = shift;
 
-	local $ENV{PGHOST} = $self->host;
 	local $ENV{PGPORT} = $self->port;
 
 	TestLib::command_fails(@_);
@@ -1900,7 +1405,7 @@ sub command_fails
 
 =item $node->command_like(...)
 
-TestLib::command_like with our connection parameters. See command_ok(...)
+TestLib::command_like with our PGPORT. See command_ok(...)
 
 =cut
 
@@ -1908,7 +1413,6 @@ sub command_like
 {
 	my $self = shift;
 
-	local $ENV{PGHOST} = $self->host;
 	local $ENV{PGPORT} = $self->port;
 
 	TestLib::command_like(@_);
@@ -1919,8 +1423,7 @@ sub command_like
 
 =item $node->command_checks_all(...)
 
-TestLib::command_checks_all with our connection parameters. See
-command_ok(...)
+TestLib::command_checks_all with our PGPORT. See command_ok(...)
 
 =cut
 
@@ -1928,7 +1431,6 @@ sub command_checks_all
 {
 	my $self = shift;
 
-	local $ENV{PGHOST} = $self->host;
 	local $ENV{PGPORT} = $self->port;
 
 	TestLib::command_checks_all(@_);
@@ -1942,20 +1444,21 @@ sub command_checks_all
 Run a command on the node, then verify that $expected_sql appears in the
 server log file.
 
+Reads the whole log file so be careful when working with large log outputs.
+The log file is truncated prior to running the command, however.
+
 =cut
 
 sub issues_sql_like
 {
 	my ($self, $cmd, $expected_sql, $test_name) = @_;
 
-	local $ENV{PGHOST} = $self->host;
 	local $ENV{PGPORT} = $self->port;
 
-	my $log_location = -s $self->logfile;
-
+	truncate $self->logfile, 0;
 	my $result = TestLib::run_log($cmd);
 	ok($result, "@$cmd exit code 0");
-	my $log = TestLib::slurp_file($self->logfile, $log_location);
+	my $log = TestLib::slurp_file($self->logfile);
 	like($log, $expected_sql, "$test_name: SQL found in server log");
 	return;
 }
@@ -1964,8 +1467,8 @@ sub issues_sql_like
 
 =item $node->run_log(...)
 
-Runs a shell command like TestLib::run_log, but with connection parameters set
-so that the command will default to connecting to this PostgresNode.
+Runs a shell command like TestLib::run_log, but with PGPORT set so
+that the command will default to connecting to this PostgresNode.
 
 =cut
 
@@ -1973,7 +1476,6 @@ sub run_log
 {
 	my $self = shift;
 
-	local $ENV{PGHOST} = $self->host;
 	local $ENV{PGPORT} = $self->port;
 
 	TestLib::run_log(@_);
@@ -2130,85 +1632,6 @@ sub wait_for_slot_catchup
 
 =pod
 
-=item $node->wait_for_subscription_sync(publisher, subname, dbname)
-
-Wait for all tables in pg_subscription_rel to complete the initial
-synchronization (i.e to be either in 'syncdone' or 'ready' state).
-
-If the publisher node is given, additionally, check if the subscriber has
-caught up to what has been committed on the primary. This is useful to
-ensure that the initial data synchronization has been completed after
-creating a new subscription.
-
-If there is no active replication connection from this peer, wait until
-poll_query_until timeout.
-
-This is not a test. It die()s on failure.
-
-=cut
-
-sub wait_for_subscription_sync
-{
-	my ($self, $publisher, $subname, $dbname) = @_;
-	my $name = $self->name;
-
-	$dbname = defined($dbname) ? $dbname : 'postgres';
-
-	# Wait for all tables to finish initial sync.
-	print "Waiting for all subscriptions in \"$name\" to synchronize data\n";
-	my $query =
-	    qq[SELECT count(1) = 0 FROM pg_subscription_rel WHERE srsubstate NOT IN ('r', 's');];
-	$self->poll_query_until($dbname, $query)
-	  or croak "timed out waiting for subscriber to synchronize data";
-
-	# Then, wait for the replication to catchup if required.
-	if (defined($publisher))
-	{
-		croak 'subscription name must be specified' unless defined($subname);
-		$publisher->wait_for_catchup($subname);
-	}
-
-	print "done\n";
-	return;
-}
-
-=pod
-
-=item $node->wait_for_log(regexp, offset)
-
-Waits for the contents of the server log file, starting at the given offset, to
-match the supplied regular expression.  Checks the entire log if no offset is
-given.  Times out after $TestLib::timeout_default seconds.
-
-If successful, returns the length of the entire log file, in bytes.
-
-=cut
-
-sub wait_for_log
-{
-	my ($self, $regexp, $offset) = @_;
-	$offset = 0 unless defined $offset;
-
-	my $max_attempts = 10 * $TestLib::timeout_default;
-	my $attempts     = 0;
-
-	while ($attempts < $max_attempts)
-	{
-		my $log = TestLib::slurp_file($self->logfile, $offset);
-
-		return $offset+length($log) if ($log =~ m/$regexp/);
-
-		# Wait 0.1 second before retrying.
-		usleep(100_000);
-
-		$attempts++;
-	}
-
-	croak "timed out waiting for match: $regexp";
-}
-
-=pod
-
 =item $node->query_hash($dbname, $query, @columns)
 
 Execute $query on $dbname, replacing any appearance of the string __COLUMNS__
@@ -2356,6 +1779,9 @@ sub pg_recvlogical_upto
 		}
 	};
 
+	$stdout =~ s/\r//g if $TestLib::windows_os;
+	$stderr =~ s/\r//g if $TestLib::windows_os;
+
 	if (wantarray)
 	{
 		return ($ret, $stdout, $stderr, $timeout);
@@ -2371,37 +1797,6 @@ sub pg_recvlogical_upto
 
 =pod
 
-=item $node->corrupt_page_checksum(self, file, page_offset)
-
-Intentionally corrupt the checksum field of one page in a file.
-The server must be stopped for this to work reliably.
-
-The file name should be specified relative to the cluster datadir.
-page_offset had better be a multiple of the cluster's block size.
-
-=cut
-
-sub corrupt_page_checksum
-{
-	my ($self, $file, $page_offset) = @_;
-	my $pgdata = $self->data_dir;
-	my $pageheader;
-
-	open my $fh, '+<', "$pgdata/$file" or die "open($file) failed: $!";
-	binmode $fh;
-	sysseek($fh, $page_offset, 0) or die "sysseek failed: $!";
-	sysread($fh, $pageheader, 24) or die "sysread failed: $!";
-	# This inverts the pd_checksum field (only); see struct PageHeaderData
-	$pageheader ^= "\0\0\0\0\0\0\0\0\xff\xff";
-	sysseek($fh, $page_offset, 0) or die "sysseek failed: $!";
-	syswrite($fh, $pageheader) or die "syswrite failed: $!";
-	close $fh;
-
-	return;
-}
-
-=pod
-
 =back
 
 =cut
