@@ -99,8 +99,8 @@ SELECT *, pg_typeof(f1) FROM
 
 -- ... unless there's context to suggest differently
 
-explain verbose select '42' union all select '43';
-explain verbose select '42' union all select 43;
+explain (verbose, costs off) select '42' union all select '43';
+explain (verbose, costs off) select '42' union all select 43;
 
 -- check materialization of an initplan reference (bug #14524)
 explain (verbose, costs off)
@@ -482,20 +482,6 @@ select val.x
   ) as val(x)
 where s.i < 10 and (select val.x) < 110;
 
--- another variant of that (bug #16213)
-explain (verbose, costs off)
-select * from
-(values
-  (3 not in (select * from (values (1), (2)) ss1)),
-  (false)
-) ss;
-
-select * from
-(values
-  (3 not in (select * from (values (1), (2)) ss1)),
-  (false)
-) ss;
-
 --
 -- Check sane behavior with nested IN SubLinks
 --
@@ -526,32 +512,6 @@ select (select q from
          ) q )
 from int4_tbl;
 
---
--- Check for sane handling of a lateral reference in a subquery's quals
--- (most of the complication here is to prevent the test case from being
--- flattened too much)
---
-explain (verbose, costs off)
-select * from
-    int4_tbl i4,
-    lateral (
-        select i4.f1 > 1 as b, 1 as id
-        from (select random() order by 1) as t1
-      union all
-        select true as b, 2 as id
-    ) as t2
-where b and f1 >= 0;
-
-select * from
-    int4_tbl i4,
-    lateral (
-        select i4.f1 > 1 as b, 1 as id
-        from (select random() order by 1) as t1
-      union all
-        select true as b, 2 as id
-    ) as t2
-where b and f1 >= 0;
-
 --
 -- Check that volatile quals aren't pushed down past a DISTINCT:
 -- nextval() should not be called more than the nominal number of times
@@ -609,6 +569,47 @@ select * from
 
 drop function tattle(x int, y int);
 
+--
+-- Test that LIMIT can be pushed to SORT through a subquery that just projects
+-- columns.  We check for that having happened by looking to see if EXPLAIN
+-- ANALYZE shows that a top-N sort was used.  We must suppress or filter away
+-- all the non-invariant parts of the EXPLAIN ANALYZE output.
+--
+create table sq_limit (pk int primary key, c1 int, c2 int);
+insert into sq_limit values
+    (1, 1, 1),
+    (2, 2, 2),
+    (3, 3, 3),
+    (4, 4, 4),
+    (5, 1, 1),
+    (6, 2, 2),
+    (7, 3, 3),
+    (8, 4, 4);
+
+create function explain_sq_limit() returns setof text language plpgsql as
+$$
+declare ln text;
+begin
+    for ln in
+        explain (analyze, summary off, timing off, costs off)
+        select * from (select pk,c2 from sq_limit order by c1,pk) as x limit 3
+    loop
+        ln := regexp_replace(ln, 'Memory: \S*',  'Memory: xxx');
+        -- this case might occur if force_parallel_mode is on:
+        ln := regexp_replace(ln, 'Worker 0:  Sort Method',  'Sort Method');
+        return next ln;
+    end loop;
+end;
+$$;
+
+select * from explain_sq_limit();
+
+select * from (select pk,c2 from sq_limit order by c1,pk) as x limit 3;
+
+drop function explain_sq_limit();
+
+drop table sq_limit;
+
 --
 -- Ensure that backward scan direction isn't propagated into
 -- expression subqueries (bug #15336)
