@@ -332,16 +332,11 @@ test_connect_ok(
 	"certificate authorization succeeds with correct client cert");
 
 # client key with wrong permissions
-SKIP:
-{
-	skip "Permissions check not enforced on Windows", 2 if ($windows_os);
-
-	test_connect_fails(
-		$common_connstr,
-		"user=ssltestuser sslcert=ssl/client.crt sslkey=ssl/client_wrongperms_tmp.key",
-		qr!\Qprivate key file "ssl/client_wrongperms_tmp.key" has group or world access\E!,
-		"certificate authorization fails because of file permissions");
-}
+test_connect_fails(
+	$common_connstr,
+	"user=ssltestuser sslcert=ssl/client.crt sslkey=ssl/client_wrongperms_tmp.key",
+	qr!\Qprivate key file "ssl/client_wrongperms_tmp.key" has group or world access\E!,
+	"certificate authorization fails because of file permissions");
 
 # client cert belonging to another user
 test_connect_fails(
@@ -355,7 +350,7 @@ test_connect_fails(
 test_connect_fails(
 	$common_connstr,
 	"user=ssltestuser sslcert=ssl/client-revoked.crt sslkey=ssl/client-revoked_tmp.key",
-	qr/SSL error|server closed the connection unexpectedly/,
+	qr/SSL error/,
 	"certificate authorization fails with revoked client cert");
 
 # intermediate client_ca.crt is provided by client, and isn't in server's ssl_ca_file
@@ -368,8 +363,7 @@ test_connect_ok(
 	"sslmode=require sslcert=ssl/client+client_ca.crt",
 	"intermediate client certificate is provided by client");
 test_connect_fails($common_connstr, "sslmode=require sslcert=ssl/client.crt",
-	qr/SSL error|server closed the connection unexpectedly/,
-	"intermediate client certificate is missing");
+	qr/SSL error/, "intermediate client certificate is missing");
 
 # clean up
 unlink("ssl/client_tmp.key", "ssl/client_wrongperms_tmp.key",
