@@ -3,7 +3,7 @@ use strict;
 use warnings;
 use PostgresNode;
 use TestLib;
-use Test::More tests => 6;
+use Test::More tests => 4;
 
 # Initialize publisher node
 my $node_publisher = get_new_node('publisher');
@@ -19,14 +19,14 @@ $node_subscriber->start;
 $node_publisher->safe_psql('postgres',
 	"CREATE TABLE tab_fk (bid int PRIMARY KEY);");
 $node_publisher->safe_psql('postgres',
-	"CREATE TABLE tab_fk_ref (id int PRIMARY KEY, junk text, bid int REFERENCES tab_fk (bid));"
+	"CREATE TABLE tab_fk_ref (id int PRIMARY KEY, bid int REFERENCES tab_fk (bid));"
 );
 
-# Setup structure on subscriber; column order intentionally different
+# Setup structure on subscriber
 $node_subscriber->safe_psql('postgres',
 	"CREATE TABLE tab_fk (bid int PRIMARY KEY);");
 $node_subscriber->safe_psql('postgres',
-	"CREATE TABLE tab_fk_ref (id int PRIMARY KEY, bid int REFERENCES tab_fk (bid), junk text);"
+	"CREATE TABLE tab_fk_ref (id int PRIMARY KEY, bid int REFERENCES tab_fk (bid));"
 );
 
 # Setup logical replication
@@ -43,10 +43,8 @@ $node_publisher->wait_for_catchup($appname);
 
 $node_publisher->safe_psql('postgres',
 	"INSERT INTO tab_fk (bid) VALUES (1);");
-# "junk" value is meant to be large enough to force out-of-line storage
 $node_publisher->safe_psql('postgres',
-	"INSERT INTO tab_fk_ref (id, bid, junk) VALUES (1, 1, repeat(pi()::text,20000));"
-);
+	"INSERT INTO tab_fk_ref (id, bid) VALUES (1, 1);");
 
 $node_publisher->wait_for_catchup($appname);
 
@@ -84,8 +82,6 @@ BEGIN
         ELSE
             RETURN NULL;
         END IF;
-    ELSIF (TG_OP = 'UPDATE') THEN
-        RETURN NULL;
     ELSE
         RAISE WARNING 'Unknown action';
         RETURN NULL;
@@ -93,7 +89,7 @@ BEGIN
 END;
 \$\$ LANGUAGE plpgsql;
 CREATE TRIGGER filter_basic_dml_trg
-    BEFORE INSERT OR UPDATE OF bid ON tab_fk_ref
+    BEFORE INSERT ON tab_fk_ref
     FOR EACH ROW EXECUTE PROCEDURE filter_basic_dml_fn();
 ALTER TABLE tab_fk_ref ENABLE REPLICA TRIGGER filter_basic_dml_trg;
 });
@@ -104,33 +100,10 @@ $node_publisher->safe_psql('postgres',
 
 $node_publisher->wait_for_catchup($appname);
 
-# The trigger should cause the insert to be skipped on subscriber
+# The row should be skipped on subscriber
 $result = $node_subscriber->safe_psql('postgres',
 	"SELECT count(*), min(bid), max(bid) FROM tab_fk_ref;");
-is($result, qq(2|1|2), 'check replica insert trigger applied on subscriber');
-
-# Update data
-$node_publisher->safe_psql('postgres',
-	"UPDATE tab_fk_ref SET bid = 2 WHERE bid = 1;");
-
-$node_publisher->wait_for_catchup($appname);
-
-# The trigger should cause the update to be skipped on subscriber
-$result = $node_subscriber->safe_psql('postgres',
-	"SELECT count(*), min(bid), max(bid) FROM tab_fk_ref;");
-is($result, qq(2|1|2), 'check replica update column trigger applied on subscriber');
-
-# Update on a column not specified in the trigger, but it will trigger
-# anyway because logical replication ships all columns in an update.
-$node_publisher->safe_psql('postgres',
-	"UPDATE tab_fk_ref SET id = 6 WHERE id = 1;");
-
-$node_publisher->wait_for_catchup($appname);
-
-$result = $node_subscriber->safe_psql('postgres',
-	"SELECT count(*), min(id), max(id) FROM tab_fk_ref;");
-is($result, qq(2|1|2),
-	'check column trigger applied even on update for other column');
+is($result, qq(2|1|2), 'check replica trigger applied on subscriber');
 
 $node_subscriber->stop('fast');
 $node_publisher->stop('fast');
