@@ -26,15 +26,6 @@ my $tmp_installdir = "$topdir/tmp_install";
 do './src/tools/msvc/config_default.pl';
 do './src/tools/msvc/config.pl' if (-f 'src/tools/msvc/config.pl');
 
-my $devnull = File::Spec->devnull;
-
-# These values are defaults that can be overridden by the calling environment
-# (see buildenv.pl processing below).  We assume that the ones listed here
-# always exist by default.  Other values may optionally be set for bincheck
-# or taptest, see set_command_env() below.
-# c.f. src/Makefile.global.in and configure.ac
-$ENV{TAR} ||= 'tar';
-
 # buildenv.pl is for specifying the build environment settings
 # it should contain lines like:
 # $ENV{PATH} = "c:/path/to/bison/bin;$ENV{PATH}";
@@ -64,14 +55,6 @@ copy("$Config/autoinc/autoinc.dll",               "src/test/regress");
 copy("$Config/regress/regress.dll",               "src/test/regress");
 copy("$Config/dummy_seclabel/dummy_seclabel.dll", "src/test/regress");
 
-# Configuration settings used by TAP tests
-$ENV{with_openssl} = $config->{openssl} ? 'yes' : 'no';
-$ENV{with_ldap} = $config->{ldap} ? 'yes' : 'no';
-$ENV{with_icu} = $config->{icu} ? 'yes' : 'no';
-$ENV{with_gssapi} = $config->{gss} ? 'yes' : 'no';
-$ENV{with_krb_srvnam} = $config->{krb_srvnam} || 'postgres';
-$ENV{with_readline} = 'no';
-
 $ENV{PATH} = "$topdir/$Config/libpq;$ENV{PATH}";
 
 if ($ENV{PERL5LIB})
@@ -116,35 +99,9 @@ exit 0;
 
 ########################################################################
 
-# Helper function for set_command_env, to set one environment command.
-sub set_single_env
+sub installcheck
 {
-	my $envname    = shift;
-	my $envdefault = shift;
-
-	# If a command is defined by the environment, just use it.
-	return if (defined($ENV{$envname}));
-
-	# Nothing is defined, so attempt to assign a default.  The command
-	# may not be in the current environment, hence check if it can be
-	# executed.
-	my $rc = system("$envdefault --version >$devnull 2>&1");
-
-	# Set the environment to the default if it exists, else leave it.
-	$ENV{$envname} = $envdefault if $rc == 0;
-	return;
-}
-
-# Set environment values for various command types.  These can be used
-# in the TAP tests.
-sub set_command_env
-{
-	set_single_env('GZIP_PROGRAM', 'gzip');
-}
-
-sub installcheck_internal
-{
-	my ($schedule, @EXTRA_REGRESS_OPTS) = @_;
+	my $schedule = shift || 'serial';
 	my @args = (
 		"../../../$Config/pg_regress/pg_regress",
 		"--dlpath=.",
@@ -154,20 +111,12 @@ sub installcheck_internal
 		"--encoding=SQL_ASCII",
 		"--no-locale");
 	push(@args, $maxconn) if $maxconn;
-	push(@args, @EXTRA_REGRESS_OPTS);
 	system(@args);
 	my $status = $? >> 8;
 	exit $status if $status;
 	return;
 }
 
-sub installcheck
-{
-	my $schedule = shift || 'serial';
-	installcheck_internal($schedule);
-	return;
-}
-
 sub check
 {
 	my $schedule = shift || 'parallel';
@@ -239,7 +188,7 @@ sub tap_check
 	  unless $config->{tap_tests};
 
 	my @flags;
-	foreach my $arg (0 .. scalar(@_) - 1)
+	foreach my $arg (0 .. scalar(@_))
 	{
 		next unless $_[$arg] =~ /^PROVE_FLAGS=(.*)/;
 		@flags = split(/\s+/, $1);
@@ -250,13 +199,12 @@ sub tap_check
 	my $dir = shift;
 	chdir $dir;
 
-	my @args = ("prove", @flags, glob("t/*.pl"));
+	my @args = ("prove", @flags, "t/*.pl");
 
 	# adjust the environment for just this test
 	local %ENV = %ENV;
 	$ENV{PERL5LIB}   = "$topdir/src/test/perl;$ENV{PERL5LIB}";
 	$ENV{PG_REGRESS} = "$topdir/$Config/pg_regress/pg_regress";
-	$ENV{REGRESS_SHLIB} = "$topdir/src/test/regress/regress.dll";
 
 	$ENV{TESTDIR} = "$dir";
 
@@ -270,8 +218,6 @@ sub bincheck
 {
 	InstallTemp();
 
-	set_command_env();
-
 	my $mstat = 0;
 
 	# Find out all the existing TAP tests by looking for t/ directories
@@ -305,9 +251,6 @@ sub taptest
 	push(@args, "$topdir/$dir");
 
 	InstallTemp();
-
-	set_command_env();
-
 	my $status = tap_check(@args);
 	exit $status if $status;
 	return;
@@ -346,7 +289,7 @@ sub mangle_plpython3
 					s/([ [{])u'/$1'/g;
 					s/def next/def __next__/g;
 					s/LANGUAGE plpython2?u/LANGUAGE plpython3u/g;
-					s/EXTENSION (\S*?)plpython2?u/EXTENSION $1plpython3u/g;
+					s/EXTENSION ([^ ]*_)*plpython2?u/EXTENSION $1plpython3u/g;
 					s/installing required extension "plpython2u"/installing required extension "plpython3u"/g;
 				  }
 				  for ($contents);
@@ -590,8 +533,7 @@ sub upgradecheck
 	$ENV{PGHOST} = 'localhost';
 	$ENV{PGPORT} ||= 50432;
 	my $tmp_root = "$topdir/src/bin/pg_upgrade/tmp_check";
-	rmtree($tmp_root);
-	mkdir $tmp_root || die $!;
+	(mkdir $tmp_root || die $!) unless -d $tmp_root;
 	my $upg_tmp_install = "$tmp_root/install";    # unshared temp install
 	print "Setting up temp install\n\n";
 	Install($upg_tmp_install, "all", $config);
@@ -603,16 +545,8 @@ sub upgradecheck
 	$ENV{PATH} = "$bindir;$ENV{PATH}";
 	my $data = "$tmp_root/data";
 	$ENV{PGDATA} = "$data.old";
-	my $outputdir          = "$tmp_root/regress";
-	my @EXTRA_REGRESS_OPTS = ("--outputdir=$outputdir");
-	mkdir "$outputdir"                || die $!;
-	mkdir "$outputdir/sql"            || die $!;
-	mkdir "$outputdir/expected"       || die $!;
-	mkdir "$outputdir/testtablespace" || die $!;
-
 	my $logdir = "$topdir/src/bin/pg_upgrade/log";
-	rmtree($logdir);
-	mkdir $logdir || die $!;
+	(mkdir $logdir || die $!) unless -d $logdir;
 	print "\nRunning initdb on old cluster\n\n";
 	standard_initdb() or exit 1;
 	print "\nStarting old cluster\n\n";
@@ -625,7 +559,7 @@ sub upgradecheck
 	generate_db('',       91, 127, '');
 
 	print "\nSetting up data for upgrading\n\n";
-	installcheck_internal('serial', @EXTRA_REGRESS_OPTS);
+	installcheck();
 
 	# now we can chdir into the source dir
 	chdir "$topdir/src/bin/pg_upgrade";
